@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,11 +20,12 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #include <cmake.h>
+#include <Hooks.h>
 #include <algorithm>
 // If <iostream> is included, put it after <stdio.h>, because it includes
 // <stdio.h>, and therefore would ignore the _WITH_GETLINE.
@@ -36,35 +37,45 @@
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <Context.h>
+#include <Variant.h>
+#include <DOM.h>
+#include <Lexer.h>
 #include <JSON.h>
-#include <Hooks.h>
 #include <Timer.h>
-#include <text.h>
+#include <FS.h>
+#include <format.h>
+#include <shared.h>
 #include <util.h>
-#include <i18n.h>
 
-extern Context context;
-
-////////////////////////////////////////////////////////////////////////////////
-Hooks::Hooks ()
-: _enabled (true)
-, _debug (0)
-{
-}
-
-////////////////////////////////////////////////////////////////////////////////
-Hooks::~Hooks ()
-{
-}
+#define STRING_HOOK_ERROR_OBJECT     "Hook Error: JSON Object '{...}' expected from hook script: {1}"
+#define STRING_HOOK_ERROR_NODESC     "Hook Error: JSON Object missing 'description' attribute from hook script: {1}"
+#define STRING_HOOK_ERROR_NOUUID     "Hook Error: JSON Object missing 'uuid' attribute from hook script: {1}"
+#define STRING_HOOK_ERROR_SYNTAX     "Hook Error: JSON syntax error in: {1}"
+#define STRING_HOOK_ERROR_JSON       "Hook Error: JSON "
+#define STRING_HOOK_ERROR_NOPARSE    "Hook Error: JSON failed to parse: "
+#define STRING_HOOK_ERROR_BAD_NUM    "Hook Error: Expected {1} JSON task(s), found {2}, in hook script: {3}"
+#define STRING_HOOK_ERROR_SAME1      "Hook Error: JSON must be for the same task: {1}, in hook script: {2}"
+#define STRING_HOOK_ERROR_SAME2      "Hook Error: JSON must be for the same task: {1} != {2}, in hook script: {3}"
+#define STRING_HOOK_ERROR_NOFEEDBACK "Hook Error: Expected feedback from failing hook script: {1}"
 
 ////////////////////////////////////////////////////////////////////////////////
 void Hooks::initialize ()
 {
-  _debug = context.config.getInteger ("debug.hooks");
+  _debug = Context::getContext ().config.getInteger ("debug.hooks");
+
+  // Scan <rc.hooks.location>
+  //      <rc.data.location>/hooks
+  Directory d;
+  if (Context::getContext ().config.has ("hooks.location"))
+  {
+    d = Directory (Context::getContext ().config.get ("hooks.location"));
+  }
+  else
+  {
+    d = Directory (Context::getContext ().config.get ("data.location"));
+    d += "hooks";
+  }
 
-  // Scan <rc.data.location>/hooks
-  Directory d (context.config.get ("data.location"));
-  d += "hooks";
   if (d.is_directory () &&
       d.readable ())
   {
@@ -73,25 +84,27 @@ void Hooks::initialize ()
 
     if (_debug >= 1)
     {
-      std::vector <std::string>::iterator i;
-      for (i = _scripts.begin (); i != _scripts.end (); ++i)
+      for (auto& i : _scripts)
       {
-        Path p (*i);
-        std::string name = p.name ();
-        if (name.substr (0, 6) == "on-add"    ||
-            name.substr (0, 9) == "on-modify" ||
-            name.substr (0, 9) == "on-launch" ||
-            name.substr (0, 7) == "on-exit")
-          context.debug ("Found hook script " + *i);
-        else
-          context.debug ("Found misnamed hook script " + *i);
+        Path p (i);
+        if (! p.is_directory ())
+        {
+          std::string name = p.name ();
+          if (name.substr (0, 6) == "on-add"    ||
+              name.substr (0, 9) == "on-modify" ||
+              name.substr (0, 9) == "on-launch" ||
+              name.substr (0, 7) == "on-exit")
+            Context::getContext ().debug ("Found hook script " + i);
+          else
+            Context::getContext ().debug ("Found misnamed hook script " + i);
+        }
       }
     }
   }
   else if (_debug >= 1)
-    context.debug ("Hook directory not readable: " + d._data);
+    Context::getContext ().debug ("Hook directory not readable: " + d._data);
 
-  _enabled = context.config.getBoolean ("hooks");
+  _enabled = Context::getContext ().config.getBoolean ("hooks");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -114,49 +127,45 @@ bool Hooks::enable (bool value)
 // - all emitted non-JSON lines are considered feedback or error messages
 //   depending on the status code.
 //
-void Hooks::onLaunch ()
+void Hooks::onLaunch () const
 {
   if (! _enabled)
     return;
 
-  context.timer_hooks.start ();
+  Timer timer;
 
   std::vector <std::string> matchingScripts = scripts ("on-launch");
   if (matchingScripts.size ())
   {
-    std::vector <std::string>::iterator script;
-    for (script = matchingScripts.begin (); script != matchingScripts.end (); ++script)
+    for (auto& script : matchingScripts)
     {
       std::vector <std::string> input;
       std::vector <std::string> output;
-      int status = callHookScript (*script, input, output);
+      int status = callHookScript (script, input, output);
 
       std::vector <std::string> outputJSON;
       std::vector <std::string> outputFeedback;
       separateOutput (output, outputJSON, outputFeedback);
 
-      assertNTasks (outputJSON, 0);
+      assertNTasks (outputJSON, 0, script);
 
       if (status == 0)
       {
-        std::vector <std::string>::iterator message;
-        for (message = outputFeedback.begin (); message != outputFeedback.end (); ++message)
-          context.footnote (*message);
+        for (auto& message : outputFeedback)
+          Context::getContext ().footnote (message);
       }
       else
       {
-        assertFeedback (outputFeedback);
-
-        std::vector <std::string>::iterator message;
-        for (message = outputFeedback.begin (); message != outputFeedback.end (); ++message)
-          context.error (*message);
+        assertFeedback (outputFeedback, script);
+        for (auto& message : outputFeedback)
+          Context::getContext ().error (message);
 
         throw 0;  // This is how hooks silently terminate processing.
       }
     }
   }
 
-  context.timer_hooks.stop ();
+  Context::getContext ().time_hooks_us += timer.total_us ();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -171,59 +180,54 @@ void Hooks::onLaunch ()
 // - all emitted non-JSON lines are considered feedback or error messages
 //   depending on the status code.
 //
-void Hooks::onExit ()
+void Hooks::onExit () const
 {
   if (! _enabled)
     return;
 
-  context.timer_hooks.start ();
+  Timer timer;
 
   std::vector <std::string> matchingScripts = scripts ("on-exit");
   if (matchingScripts.size ())
   {
     // Get the set of changed tasks.
     std::vector <Task> tasks;
-    context.tdb2.get_changes (tasks);
+    Context::getContext ().tdb2.get_changes (tasks);
 
     // Convert to a vector of strings.
     std::vector <std::string> input;
-    std::vector <Task>::const_iterator t;
-    for (t = tasks.begin (); t != tasks.end (); ++t)
-      input.push_back (t->composeJSON ());
+    for (auto& t : tasks)
+      input.push_back (t.composeJSON ());
 
     // Call the hook scripts, with the invariant input.
-    std::vector <std::string>::iterator script;
-    for (script = matchingScripts.begin (); script != matchingScripts.end (); ++script)
+    for (auto& script : matchingScripts)
     {
       std::vector <std::string> output;
-      int status = callHookScript (*script, input, output);
+      int status = callHookScript (script, input, output);
 
       std::vector <std::string> outputJSON;
       std::vector <std::string> outputFeedback;
       separateOutput (output, outputJSON, outputFeedback);
 
-      assertNTasks (outputJSON, 0);
+      assertNTasks (outputJSON, 0, script);
 
       if (status == 0)
       {
-        std::vector <std::string>::iterator message;
-        for (message = outputFeedback.begin (); message != outputFeedback.end (); ++message)
-          context.footnote (*message);
+        for (auto& message : outputFeedback)
+          Context::getContext ().footnote (message);
       }
       else
       {
-        assertFeedback (outputFeedback);
-
-        std::vector <std::string>::iterator message;
-        for (message = outputFeedback.begin (); message != outputFeedback.end (); ++message)
-          context.error (*message);
+        assertFeedback (outputFeedback, script);
+        for (auto& message : outputFeedback)
+          Context::getContext ().error (message);
 
         throw 0;  // This is how hooks silently terminate processing.
       }
     }
   }
 
-  context.timer_hooks.stop ();
+  Context::getContext ().time_hooks_us += timer.total_us ();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -238,12 +242,12 @@ void Hooks::onExit ()
 // - all emitted non-JSON lines are considered feedback or error messages
 //   depending on the status code.
 //
-void Hooks::onAdd (Task& task)
+void Hooks::onAdd (Task& task) const
 {
   if (! _enabled)
     return;
 
-  context.timer_hooks.start ();
+  Timer timer;
 
   std::vector <std::string> matchingScripts = scripts ("on-add");
   if (matchingScripts.size ())
@@ -253,11 +257,10 @@ void Hooks::onAdd (Task& task)
     input.push_back (task.composeJSON ());
 
     // Call the hook scripts.
-    std::vector <std::string>::iterator script;
-    for (script = matchingScripts.begin (); script != matchingScripts.end (); ++script)
+    for (auto& script : matchingScripts)
     {
       std::vector <std::string> output;
-      int status = callHookScript (*script, input, output);
+      int status = callHookScript (script, input, output);
 
       std::vector <std::string> outputJSON;
       std::vector <std::string> outputFeedback;
@@ -265,24 +268,21 @@ void Hooks::onAdd (Task& task)
 
       if (status == 0)
       {
-        assertNTasks    (outputJSON, 1);
-        assertValidJSON (outputJSON);
-        assertSameTask  (outputJSON, task);
+        assertNTasks    (outputJSON, 1, script);
+        assertValidJSON (outputJSON, script);
+        assertSameTask  (outputJSON, task, script);
 
         // Propagate forward to the next script.
         input[0] = outputJSON[0];
 
-        std::vector <std::string>::iterator message;
-        for (message = outputFeedback.begin (); message != outputFeedback.end (); ++message)
-          context.footnote (*message);
+        for (auto& message : outputFeedback)
+          Context::getContext ().footnote (message);
       }
       else
       {
-        assertFeedback (outputFeedback);
-
-        std::vector <std::string>::iterator message;
-        for (message = outputFeedback.begin (); message != outputFeedback.end (); ++message)
-          context.error (*message);
+        assertFeedback (outputFeedback, script);
+        for (auto& message : outputFeedback)
+          Context::getContext ().error (message);
 
         throw 0;  // This is how hooks silently terminate processing.
       }
@@ -292,7 +292,7 @@ void Hooks::onAdd (Task& task)
     task = Task (input[0]);
   }
 
-  context.timer_hooks.stop ();
+  Context::getContext ().time_hooks_us += timer.total_us ();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -308,12 +308,12 @@ void Hooks::onAdd (Task& task)
 // - all emitted non-JSON lines are considered feedback or error messages
 //   depending on the status code.
 //
-void Hooks::onModify (const Task& before, Task& after)
+void Hooks::onModify (const Task& before, Task& after) const
 {
   if (! _enabled)
     return;
 
-  context.timer_hooks.start ();
+  Timer timer;
 
   std::vector <std::string> matchingScripts = scripts ("on-modify");
   if (matchingScripts.size ())
@@ -324,11 +324,10 @@ void Hooks::onModify (const Task& before, Task& after)
     input.push_back (after.composeJSON ());  // [line 1] modified
 
     // Call the hook scripts.
-    std::vector <std::string>::iterator script;
-    for (script = matchingScripts.begin (); script != matchingScripts.end (); ++script)
+    for (auto& script : matchingScripts)
     {
       std::vector <std::string> output;
-      int status = callHookScript (*script, input, output);
+      int status = callHookScript (script, input, output);
 
       std::vector <std::string> outputJSON;
       std::vector <std::string> outputFeedback;
@@ -336,24 +335,21 @@ void Hooks::onModify (const Task& before, Task& after)
 
       if (status == 0)
       {
-        assertNTasks    (outputJSON, 1);
-        assertValidJSON (outputJSON);
-        assertSameTask  (outputJSON, before);
+        assertNTasks    (outputJSON, 1, script);
+        assertValidJSON (outputJSON, script);
+        assertSameTask  (outputJSON, before, script);
 
         // Propagate accepted changes forward to the next script.
         input[1] = outputJSON[0];
 
-        std::vector <std::string>::iterator message;
-        for (message = outputFeedback.begin (); message != outputFeedback.end (); ++message)
-          context.footnote (*message);
+        for (auto& message : outputFeedback)
+          Context::getContext ().footnote (message);
       }
       else
       {
-        assertFeedback (outputFeedback);
-
-        std::vector <std::string>::iterator message;
-        for (message = outputFeedback.begin (); message != outputFeedback.end (); ++message)
-          context.error (*message);
+        assertFeedback (outputFeedback, script);
+        for (auto& message : outputFeedback)
+          Context::getContext ().error (message);
 
         throw 0;  // This is how hooks silently terminate processing.
       }
@@ -362,27 +358,26 @@ void Hooks::onModify (const Task& before, Task& after)
     after = Task (input[1]);
   }
 
-  context.timer_hooks.stop ();
+  Context::getContext ().time_hooks_us += timer.total_us ();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::vector <std::string> Hooks::list ()
+std::vector <std::string> Hooks::list () const
 {
   return _scripts;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::vector <std::string> Hooks::scripts (const std::string& event)
+std::vector <std::string> Hooks::scripts (const std::string& event) const
 {
   std::vector <std::string> matching;
-  std::vector <std::string>::iterator i;
-  for (i = _scripts.begin (); i != _scripts.end (); ++i)
+  for (const auto& i : _scripts)
   {
-    if (i->find ("/" + event) != std::string::npos)
+    if (i.find ("/" + event) != std::string::npos)
     {
-      File script (*i);
+      File script (i);
       if (script.executable ())
-        matching.push_back (*i);
+        matching.push_back (i);
     }
   }
 
@@ -395,13 +390,12 @@ void Hooks::separateOutput (
   std::vector <std::string>& json,
   std::vector <std::string>& feedback) const
 {
-  std::vector <std::string>::const_iterator i;
-  for (i = output.begin (); i != output.end (); ++i)
+  for (auto& i : output)
   {
-    if (isJSON (*i))
-      json.push_back (*i);
+    if (isJSON (i))
+      json.push_back (i);
     else
-      feedback.push_back (*i);
+      feedback.push_back (i);
   }
 }
 
@@ -414,114 +408,127 @@ bool Hooks::isJSON (const std::string& input) const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void Hooks::assertValidJSON (const std::vector <std::string>& input) const
+void Hooks::assertValidJSON (
+  const std::vector <std::string>& input,
+  const std::string& script) const
 {
-  std::vector <std::string>::const_iterator i;
-  for (i = input.begin (); i != input.end (); i++)
+  for (auto& i : input)
   {
-    if (i->length () < 3 ||
-        (*i)[0] != '{'   ||
-        (*i)[i->length () - 1] != '}')
+    if (i.length () < 3 ||
+        i[0] != '{'     ||
+        i[i.length () - 1] != '}')
     {
-      context.error (STRING_HOOK_ERROR_OBJECT);
+      Context::getContext ().error (format (STRING_HOOK_ERROR_OBJECT, Path (script).name ()));
       throw 0;
     }
 
     try
     {
-      json::value* root = json::parse (*i);
+      json::value* root = json::parse (i);
       if (root->type () != json::j_object)
       {
-        context.error (STRING_HOOK_ERROR_OBJECT);
+        Context::getContext ().error (format (STRING_HOOK_ERROR_OBJECT, Path (script).name ()));
         throw 0;
       }
 
       if (((json::object*)root)->_data.find ("description") == ((json::object*)root)->_data.end ())
       {
-        context.error (STRING_HOOK_ERROR_NODESC);
+        Context::getContext ().error (format (STRING_HOOK_ERROR_NODESC, Path (script).name ()));
         throw 0;
       }
 
       if (((json::object*)root)->_data.find ("uuid") == ((json::object*)root)->_data.end ())
       {
-        context.error (STRING_HOOK_ERROR_NOUUID);
+        Context::getContext ().error (format (STRING_HOOK_ERROR_NOUUID, Path (script).name ()));
         throw 0;
       }
+
+      delete root;
     }
 
     catch (const std::string& e)
     {
-      context.error (format (STRING_HOOK_ERROR_SYNTAX, *i));
+      Context::getContext ().error (format (STRING_HOOK_ERROR_SYNTAX, i));
       if (_debug)
-        context.error (STRING_HOOK_ERROR_JSON + e);
+        Context::getContext ().error (STRING_HOOK_ERROR_JSON + e);
       throw 0;
     }
 
     catch (...)
     {
-      context.error (STRING_HOOK_ERROR_NOPARSE + *i);
+      Context::getContext ().error (STRING_HOOK_ERROR_NOPARSE + i);
       throw 0;
     }
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void Hooks::assertNTasks (const std::vector <std::string>& input, unsigned int n) const
+void Hooks::assertNTasks (
+  const std::vector <std::string>& input,
+  unsigned int n,
+  const std::string& script) const
 {
   if (input.size () != n)
   {
-    context.error (format (STRING_HOOK_ERROR_BAD_NUM, n, (int) input.size ()));
+    Context::getContext ().error (format (STRING_HOOK_ERROR_BAD_NUM, n, (int) input.size (), Path (script).name ()));
     throw 0;
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void Hooks::assertSameTask (const std::vector <std::string>& input, const Task& task) const
+void Hooks::assertSameTask (
+  const std::vector <std::string>& input,
+  const Task& task,
+  const std::string& script) const
 {
   std::string uuid = task.get ("uuid");
 
-  std::vector <std::string>::const_iterator i;
-  for (i = input.begin (); i != input.end (); i++)
+  for (auto& i : input)
   {
-    json::object* root_obj = (json::object*)json::parse (*i);
+    json::object* root_obj = (json::object*)json::parse (i);
 
     // If there is no UUID at all.
-    json_object_iter u = root_obj->_data.find ("uuid");
+    auto u = root_obj->_data.find ("uuid");
     if (u == root_obj->_data.end ()          ||
         u->second->type () != json::j_string)
     {
-      context.error (format (STRING_HOOK_ERROR_SAME1, uuid));
+      Context::getContext ().error (format (STRING_HOOK_ERROR_SAME1, uuid, Path (script).name ()));
       throw 0;
     }
 
     json::string* up = (json::string*) u->second;
-    std::string json_uuid = json::decode (unquoteText (up->dump ()));
+    auto text = up->dump ();
+    Lexer::dequote (text);
+    std::string json_uuid = json::decode (text);
     if (json_uuid != uuid)
     {
-      context.error (format (STRING_HOOK_ERROR_SAME2, uuid, json_uuid));
+      Context::getContext ().error (format (STRING_HOOK_ERROR_SAME2, uuid, json_uuid, Path (script).name ()));
       throw 0;
     }
+
+    delete root_obj;
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void Hooks::assertFeedback (const std::vector <std::string>& input) const
+void Hooks::assertFeedback (
+  const std::vector <std::string>& input,
+  const std::string& script) const
 {
   bool foundSomething = false;
-  std::vector <std::string>::const_iterator i;
-  for (i = input.begin (); i != input.end (); ++i)
-    if (nontrivial (*i))
+  for (auto& i : input)
+    if (nontrivial (i))
       foundSomething = true;
 
   if (! foundSomething)
   {
-    context.error (STRING_HOOK_ERROR_NOFEEDBACK);
+    Context::getContext ().error (format (STRING_HOOK_ERROR_NOFEEDBACK, Path (script).name ()));
     throw 0;
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::vector <std::string>& Hooks::buildHookScriptArgs (std::vector <std::string>& args)
+std::vector <std::string>& Hooks::buildHookScriptArgs (std::vector <std::string>& args) const
 {
   Variant v;
 
@@ -529,17 +536,17 @@ std::vector <std::string>& Hooks::buildHookScriptArgs (std::vector <std::string>
   args.push_back ("api:2");
 
   // Command line Taskwarrior was called with.
-  context.dom.get ("context.args", v);
+  getDOM ("context.args", v);
   args.push_back ("args:" + std::string (v));
 
   // Command to be executed.
-  args.push_back ("command:" + context.cli.getCommand ());
+  args.push_back ("command:" + Context::getContext ().cli2.getCommand ());
 
   // rc file used after applying all overrides.
-  args.push_back ("rc:" + context.rc_file._data);
+  args.push_back ("rc:" + Context::getContext ().rc_file._data);
 
   // Directory containing *.data files.
-  args.push_back ("data:" + context.data_dir._data);
+  args.push_back ("data:" + Context::getContext ().data_dir._data);
 
   // Taskwarrior version, same as returned by "task --version"
   args.push_back ("version:" + std::string(VERSION));
@@ -551,31 +558,29 @@ std::vector <std::string>& Hooks::buildHookScriptArgs (std::vector <std::string>
 int Hooks::callHookScript (
   const std::string& script,
   const std::vector <std::string>& input,
-  std::vector <std::string>& output)
+  std::vector <std::string>& output) const
 {
   if (_debug >= 1)
-    context.debug ("Hook: Calling " + script);
+    Context::getContext ().debug ("Hook: Calling " + script);
 
   if (_debug >= 2)
   {
-    context.debug ("Hook: input");
-    std::vector <std::string>::const_iterator i;
-    for (i = input.begin (); i != input.end (); ++i)
-      context.debug ("  " + *i);
+    Context::getContext ().debug ("Hook: input");
+    for (const auto& i : input)
+      Context::getContext ().debug ("  " + i);
   }
 
   std::string inputStr;
-  std::vector <std::string>::const_iterator i;
-  for (i = input.begin (); i != input.end (); ++i)
-    inputStr += *i + "\n";
+  for (const auto& i : input)
+    inputStr += i + "\n";
 
   std::vector <std::string> args;
   buildHookScriptArgs (args);
   if (_debug >= 2)
   {
-    context.debug ("Hooks: args");
-    for (auto arg: args)
-      context.debug ("  " + arg);
+    Context::getContext ().debug ("Hooks: args");
+    for (const auto& arg: args)
+      Context::getContext ().debug ("  " + arg);
   }
 
   // Measure time for each hook if running in debug
@@ -583,26 +588,24 @@ int Hooks::callHookScript (
   std::string outputStr;
   if (_debug >= 2)
   {
-    Timer timer_per_hook("Hooks::execute (" + script + ")");
-    timer_per_hook.start();
-
+    Timer timer;
     status = execute (script, args, inputStr, outputStr);
+    Context::getContext ().debugTiming (format ("Hooks::execute ({1})", script), timer);
   }
   else
     status = execute (script, args, inputStr, outputStr);
 
-  split (output, outputStr, '\n');
+  output = split (outputStr, '\n');
 
   if (_debug >= 2)
   {
-    context.debug ("Hook: output");
-    std::vector <std::string>::iterator i;
-    for (i = output.begin (); i != output.end (); ++i)
-      if (*i != "")
-        context.debug ("  " + *i);
+    Context::getContext ().debug ("Hook: output");
+    for (const auto& i : output)
+      if (i != "")
+        Context::getContext ().debug ("  " + i);
 
-    context.debug (format ("Hook: Completed with status {1}", status));
-    context.debug (" "); // Blank line
+    Context::getContext ().debug (format ("Hook: Completed with status {1}", status));
+    Context::getContext ().debug (" "); // Blank line
   }
 
   return status;
