@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -32,12 +32,14 @@
 #include <string>
 #include <stdio.h>
 #include <time.h>
+#include <JSON.h>
 
-class Task : public std::map <std::string, std::string>
+class Task
 {
 public:
   static std::string defaultProject;
   static std::string defaultDue;
+  static std::string defaultScheduled;
   static bool searchCaseSensitive;
   static bool regex;
   static std::map <std::string, std::string> attributes;  // name -> type
@@ -48,22 +50,18 @@ public:
   static float urgencyScheduledCoefficient;
   static float urgencyWaitingCoefficient;
   static float urgencyBlockedCoefficient;
-  static float urgencyInheritCoefficient;
   static float urgencyAnnotationsCoefficient;
   static float urgencyTagsCoefficient;
-  static float urgencyNextCoefficient;
   static float urgencyDueCoefficient;
   static float urgencyBlockingCoefficient;
   static float urgencyAgeCoefficient;
   static float urgencyAgeMax;
 
 public:
-  Task ();                       // Default constructor
-  Task (const Task&);            // Copy constructor
-  Task& operator= (const Task&); // Assignment operator
-  bool operator== (const Task&); // Comparison operator
-  Task (const std::string&);     // Parse
-  ~Task ();                      // Destructor
+  Task () = default;
+  bool operator== (const Task&);
+  Task (const std::string&);
+  Task (const json::object*);
 
   void parse (const std::string&);
   std::string composeF4 () const;
@@ -76,14 +74,13 @@ public:
   enum dateState {dateNotDue, dateAfterToday, dateLaterToday, dateEarlierToday, dateBeforeToday};
 
   // Public data.
-  int id;
-  float urgency_value;
-  bool recalc_urgency;
-
-  bool is_blocked;
-  bool is_blocking;
-
-  int annotation_count;
+  std::map <std::string, std::string> data {};
+  int id                                   {0};
+  float urgency_value                      {0.0};
+  bool recalc_urgency                      {true};
+  bool is_blocked                          {false};
+  bool is_blocking                         {false};
+  int annotation_count                     {0};
 
   // Series of helper functions.
   static status textToStatus (const std::string&);
@@ -92,6 +89,7 @@ public:
   void setAsNow (const std::string&);
   bool has (const std::string&) const;
   std::vector <std::string> all ();
+  const std::string identifier (bool shortened = false) const;
   const std::string get (const std::string&) const;
   const std::string& get_ref (const std::string&) const;
   int get_int (const std::string&) const;
@@ -110,40 +108,48 @@ public:
   bool is_duetomorrow () const;
   bool is_dueweek () const;
   bool is_duemonth () const;
+  bool is_duequarter () const;
   bool is_dueyear () const;
   bool is_overdue () const;
+  bool is_udaPresent () const;
+  bool is_orphanPresent () const;
 #endif
 
   status getStatus () const;
   void setStatus (status);
 
+#ifdef PRODUCT_TASKWARRIOR
   dateState getDateState (const std::string&) const;
+#endif
 
   int getTagCount () const;
   bool hasTag (const std::string&) const;
   void addTag (const std::string&);
   void addTags (const std::vector <std::string>&);
-  void getTags (std::vector<std::string>&) const;
+  std::vector <std::string> getTags () const;
   void removeTag (const std::string&);
 
+  int getAnnotationCount () const;
   bool hasAnnotations () const;
-  void getAnnotations (std::map <std::string, std::string>&) const;
+  std::map <std::string, std::string> getAnnotations () const;
   void setAnnotations (const std::map <std::string, std::string>&);
   void addAnnotation (const std::string&);
   void removeAnnotations ();
 
 #ifdef PRODUCT_TASKWARRIOR
   void addDependency (int);
+#endif
   void addDependency (const std::string&);
+#ifdef PRODUCT_TASKWARRIOR
   void removeDependency (int);
   void removeDependency (const std::string&);
-  void getDependencies (std::vector <int>&) const;
-  void getDependencies (std::vector <std::string>&) const;
+  std::vector <int>         getDependencyIDs () const;
+  std::vector <std::string> getDependencyUUIDs () const;
+  std::vector <Task>        getDependencyTasks () const;
 
-  void getUDAs (std::vector <std::string>&) const;
-  void getUDAOrphans (std::vector <std::string>&) const;
+  std::vector <std::string> getUDAOrphanUUIDs () const;
 
-  void substitute (const std::string&, const std::string&, bool);
+  void substitute (const std::string&, const std::string&, const std::string&);
 #endif
 
   void validate (bool applyDefault = true);
@@ -151,30 +157,32 @@ public:
   float urgency_c () const;
   float urgency ();
 
+#ifdef PRODUCT_TASKWARRIOR
   enum modType {modReplace, modPrepend, modAppend, modAnnotate};
   void modify (modType, bool text_required = false);
+#endif
 
 private:
   int determineVersion (const std::string&);
   void parseJSON (const std::string&);
+  void parseJSON (const json::object*);
   void parseLegacy (const std::string&);
   void validate_before (const std::string&, const std::string&);
   const std::string encode (const std::string&) const;
   const std::string decode (const std::string&) const;
 
 public:
-  float urgency_project () const;
-  float urgency_active () const;
-  float urgency_scheduled () const;
-  float urgency_waiting () const;
-  float urgency_blocked () const;
-  float urgency_inherit () const;
+  float urgency_project     () const;
+  float urgency_active      () const;
+  float urgency_scheduled   () const;
+  float urgency_waiting     () const;
+  float urgency_blocked     () const;
+  float urgency_inherit     () const;
   float urgency_annotations () const;
-  float urgency_tags () const;
-  float urgency_next () const;
-  float urgency_due () const;
-  float urgency_blocking () const;
-  float urgency_age () const;
+  float urgency_tags        () const;
+  float urgency_due         () const;
+  float urgency_blocking    () const;
+  float urgency_age         () const;
 };
 
 #endif
