@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,49 +20,36 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #include <cmake.h>
+#include <ColDepends.h>
 #include <algorithm>
 #include <Context.h>
-#include <ColDepends.h>
-#include <text.h>
+#include <shared.h>
+#include <format.h>
 #include <utf8.h>
-#include <i18n.h>
 #include <main.h>
+#include <stdlib.h>
 
-extern Context context;
+#define STRING_COLUMN_LABEL_DEP "Depends"
 
 ////////////////////////////////////////////////////////////////////////////////
 ColumnDepends::ColumnDepends ()
 {
-  _name  = "depends";
-  _type  = "string";
-  _style = "list";
-  _label = STRING_COLUMN_LABEL_DEP;
+  _name      = "depends";
+  _style     = "list";
+  _label     = STRING_COLUMN_LABEL_DEP;
+  _styles    = {"list",
+                "count",
+                "indicator"};
+  _examples  = {"1 2 10",
+                "[3]",
+                Context::getContext ().config.get ("dependency.indicator")};
 
-  _styles.push_back ("list");
-  _styles.push_back ("count");
-  _styles.push_back ("indicator");
-
-  _examples.push_back ("1 2 10");
-  _examples.push_back ("[3]");
-  _examples.push_back (context.config.get ("dependency.indicator"));
-
-  _hyphenate = context.config.getBoolean ("hyphenate");
-}
-
-////////////////////////////////////////////////////////////////////////////////
-ColumnDepends::~ColumnDepends ()
-{
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool ColumnDepends::validate (std::string& value)
-{
-  return true;
+  _hyphenate = false;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -70,47 +57,51 @@ bool ColumnDepends::validate (std::string& value)
 // Note that you can not determine which gets called first.
 void ColumnDepends::setStyle (const std::string& value)
 {
-  _style = value;
+  Column::setStyle (value);
 
-       if (_style == "indicator" && _label == STRING_COLUMN_LABEL_DEP) _label = _label.substr (0, context.config.get ("dependency.indicator").length ());
-  else if (_style == "count"     && _label == STRING_COLUMN_LABEL_DEP) _label = STRING_COLUMN_LABEL_DEP_S;
+       if (_style == "indicator" && _label == STRING_COLUMN_LABEL_DEP) _label = _label.substr (0, Context::getContext ().config.get ("dependency.indicator").length ());
+  else if (_style == "count"     && _label == STRING_COLUMN_LABEL_DEP) _label = "Dep";
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set the minimum and maximum widths for the value.
 void ColumnDepends::measure (Task& task, unsigned int& minimum, unsigned int& maximum)
 {
-  std::vector <Task> blocking;
-  dependencyGetBlocking (task, blocking);
-
-       if (_style == "indicator") minimum = maximum = utf8_width (context.config.get ("dependency.indicator"));
-  else if (_style == "count")     minimum = maximum = 2 + format ((int) blocking.size ()).length ();
-  else if (_style == "default" ||
-           _style == "list")
+  minimum = maximum = 0;
+  if (task.has (_name))
   {
-    minimum = maximum = 0;
-    if (task.has ("depends"))
+    if (_style == "indicator")
     {
-      std::vector <int> blocking_ids;
-      std::vector <Task>::iterator i;
-      for (i = blocking.begin (); i != blocking.end (); ++i)
-        blocking_ids.push_back (i->id);
+      minimum = maximum = utf8_width (Context::getContext ().config.get ("dependency.indicator"));
+    }
 
-      std::string all;
-      join (all, " ", blocking_ids);
+    else if (_style == "count")
+    {
+      minimum = maximum = 2 + format ((int) dependencyGetBlocking (task).size ()).length ();
+    }
+
+    else if (_style == "default" ||
+             _style == "list")
+    {
+      minimum = maximum = 0;
+      auto blocking = dependencyGetBlocking (task);
+
+      std::vector <int> blocking_ids;
+      for (auto& i : blocking)
+        blocking_ids.push_back (i.id);
+
+      auto all = join (" ", blocking_ids);
       maximum = all.length ();
 
       unsigned int length;
-      for (i = blocking.begin (); i != blocking.end (); ++i)
+      for (auto& i : blocking)
       {
-        length = format (i->id).length ();
+        length = format (i.id).length ();
         if (length > minimum)
           minimum = length;
       }
     }
   }
-  else
-    throw format (STRING_COLUMN_BAD_FORMAT, _name, _style);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -124,38 +115,53 @@ void ColumnDepends::render (
   {
     if (_style == "indicator")
     {
-      lines.push_back (
-        color.colorize (
-          rightJustify (context.config.get ("dependency.indicator"), width)));
-      return;
+      renderStringRight (lines, width, color, Context::getContext ().config.get ("dependency.indicator"));
     }
 
-    std::vector <Task> blocking;
-    dependencyGetBlocking (task, blocking);
-
-    if (_style == "count")
+    else if (_style == "count")
     {
-      lines.push_back (
-        color.colorize (
-          rightJustify ("[" + format ((int)blocking.size ()) + "]", width)));
+      renderStringRight (lines, width, color, '[' + format (static_cast <int>(dependencyGetBlocking (task).size ())) + ']');
     }
+
     else if (_style == "default" ||
              _style == "list")
     {
-      std::vector <int> blocking_ids;
-      std::vector <Task>::iterator t;
-      for (t = blocking.begin (); t != blocking.end (); ++t)
-        blocking_ids.push_back (t->id);
+      auto blocking = dependencyGetBlocking (task);
 
-      std::string combined;
-      join (combined, " ", blocking_ids);
+      std::vector <int> blocking_ids;
+      for (const auto& t : blocking)
+        blocking_ids.push_back (t.id);
+
+      auto combined = join (" ", blocking_ids);
 
       std::vector <std::string> all;
       wrapText (all, combined, width, _hyphenate);
 
-      std::vector <std::string>::iterator i;
-      for (i = all.begin (); i != all.end (); ++i)
-        lines.push_back (color.colorize (leftJustify (*i, width)));
+      for (const auto& i : all)
+        renderStringLeft (lines, width, color, i);
+    }
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+void ColumnDepends::modify (Task& task, const std::string& value)
+{
+  // Apply or remove dendencies in turn.
+  for (auto& dep : split (value, ','))
+  {
+    if (dep[0] == '-')
+    {
+      if (dep.length () == 37)
+        task.removeDependency (dep.substr (1));
+      else
+        task.removeDependency (strtol (dep.substr (1).c_str (), nullptr, 10));
+    }
+    else
+    {
+      if (dep.length () == 36)
+        task.addDependency (dep);
+      else
+        task.addDependency (strtol (dep.c_str (), nullptr, 10));
     }
   }
 }
