@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,44 +20,19 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
-#include <stdlib.h>
-#include <Context.h>
+#include <cmake.h>
 #include <ColDue.h>
-#include <Date.h>
-#include <Duration.h>
-#include <text.h>
-#include <i18n.h>
-
-extern Context context;
 
 ////////////////////////////////////////////////////////////////////////////////
 ColumnDue::ColumnDue ()
 {
-  _name      = "due";
-  _label     = STRING_COLUMN_LABEL_DUE;
-
-  _styles.push_back ("countdown");
-
-  Date now;
-  now += 125;
-  _examples.push_back (Duration (now - Date ()).formatCompact ());
-}
-
-////////////////////////////////////////////////////////////////////////////////
-ColumnDue::~ColumnDue ()
-{
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool ColumnDue::validate (std::string& value)
-{
-  return ColumnDate::validate (value);
+  _name       = "due";
+  _modifiable = true;
+  _label      = "Due";
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -66,10 +40,10 @@ bool ColumnDue::validate (std::string& value)
 // Note that you can not determine which gets called first.
 void ColumnDue::setStyle (const std::string& value)
 {
-  _style = value;
+  Column::setStyle (value);
 
-  if (_style == "countdown" && _label == STRING_COLUMN_LABEL_DUE)
-    _label = STRING_COLUMN_LABEL_COUNT;
+  if (_style == "countdown" && _label == "Due")
+    _label = "Count";
 }
 
 ////////////////////////////////////////////////////////////////////////////////
