@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,55 +20,35 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
-#include <math.h>
-#include <Context.h>
+#include <cmake.h>
 #include <ColParent.h>
-#include <text.h>
-#include <i18n.h>
-
-extern Context context;
+#include <format.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 ColumnParent::ColumnParent ()
 {
   _name       = "parent";
-  _type       = "string";
   _style      = "long";
-  _label      = STRING_COLUMN_LABEL_PARENT;
+  _label      = "Parent task";
   _modifiable = false;
-
-  _styles.push_back ("long");
-  _styles.push_back ("short");
-
-  _examples.push_back ("f30cb9c3-3fc0-483f-bfb2-3bf134f00694");
-  _examples.push_back ("34f00694");
-}
-
-////////////////////////////////////////////////////////////////////////////////
-ColumnParent::~ColumnParent ()
-{
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool ColumnParent::validate (std::string& value)
-{
-  return true;
+  _styles     = {"long", "short"};
+  _examples   = {"f30cb9c3-3fc0-483f-bfb2-3bf134f00694", "f30cb9c3"};
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set the minimum and maximum widths for the value.
-void ColumnParent::measure (Task&, unsigned int& minimum, unsigned int& maximum)
+void ColumnParent::measure (Task& task, unsigned int& minimum, unsigned int& maximum)
 {
-       if (_style == "default" || _style == "long") minimum = maximum = 36;
-  else if (_style == "short")                       minimum = maximum = 8;
-  else
-    throw format (STRING_COLUMN_BAD_FORMAT, _name, _style);
+  minimum = maximum = 0;
+  if (task.has (_name))
+  {
+         if (_style == "default" || _style == "long") minimum = maximum = 36;
+    else if (_style == "short")                       minimum = maximum = 8;
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -79,14 +58,17 @@ void ColumnParent::render (
   int width,
   Color& color)
 {
-  // f30cb9c3-3fc0-483f-bfb2-3bf134f00694  default
-  //                             34f00694  short
-  if (_style == "default" ||
-      _style == "long")
-    lines.push_back (color.colorize (leftJustify (task.get (_name), width)));
+  if (task.has (_name))
+  {
+    // f30cb9c3-3fc0-483f-bfb2-3bf134f00694  default
+    // f30cb9c3                              short
+    if (_style == "default" ||
+        _style == "long")
+      renderStringLeft (lines, width, color, task.get(_name));
 
-  else if (_style == "short")
-    lines.push_back (color.colorize (leftJustify (task.get (_name).substr (28), width)));
+    else if (_style == "short")
+      renderStringLeft (lines, width, color, task.get (_name).substr (0, 8));
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
