@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,48 +20,33 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
+#include <cmake.h>
+#include <ColRecur.h>
 #include <Context.h>
 #include <Duration.h>
-#include <ColRecur.h>
-#include <text.h>
-#include <i18n.h>
+#include <Eval.h>
+#include <Variant.h>
+#include <Lexer.h>
+#include <Filter.h>
+#include <shared.h>
+#include <format.h>
+#include <utf8.h>
 
-extern Context context;
+extern Task& contextTask;
 
 ////////////////////////////////////////////////////////////////////////////////
 ColumnRecur::ColumnRecur ()
 {
-  _name  = "recur";
-
-  // This is 'string', and not 'duration' to force the value to be stored as a
-  // raw duration, so that it can be reevaluated every time.
-  _type  = "string";
-
-  _style = "duration";
-  _label = STRING_COLUMN_LABEL_RECUR;
-
-  _styles.push_back ("duration");
-  _styles.push_back ("indicator");
-
-  _examples.push_back ("weekly");
-  _examples.push_back (context.config.get ("recurrence.indicator"));
-}
-
-////////////////////////////////////////////////////////////////////////////////
-ColumnRecur::~ColumnRecur ()
-{
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool ColumnRecur::validate (std::string& value)
-{
-  return true;
+  _name       = "recur";
+  _style      = "duration";
+  _label      = "Recur";
+  _modifiable = true;
+  _styles     = {"duration", "indicator"};
+  _examples   = {"weekly", Context::getContext ().config.get ("recurrence.indicator")};
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -70,28 +54,29 @@ bool ColumnRecur::validate (std::string& value)
 // Note that you can not determine which gets called first.
 void ColumnRecur::setStyle (const std::string& value)
 {
-  _style = value;
+  Column::setStyle (value);
 
-  if (_style == "indicator" && _label == STRING_COLUMN_LABEL_RECUR)
-    _label = _label.substr (0, context.config.get ("recurrence.indicator").length ());
+  if (_style == "indicator" && _label == "Recur")
+    _label = _label.substr (0, Context::getContext ().config.get ("recurrence.indicator").length ());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set the minimum and maximum widths for the value.
 void ColumnRecur::measure (Task& task, unsigned int& minimum, unsigned int& maximum)
 {
-  if (_style == "default" ||
-      _style == "duration")
+  minimum = maximum = 0;
+  if (task.has (_name))
   {
-    minimum = maximum = Duration (task.get ("recur")).formatCompact ().length ();
+    if (_style == "default" ||
+        _style == "duration")
+    {
+      minimum = maximum = Duration (task.get (_name)).formatISO ().length ();
+    }
+    else if (_style == "indicator")
+    {
+      minimum = maximum = utf8_width (Context::getContext ().config.get ("recurrence.indicator"));
+    }
   }
-  else if (_style == "indicator")
-  {
-    if (task.has (_name))
-      minimum = maximum = context.config.get ("recurrence.indicator").length ();
-  }
-  else
-    throw format (STRING_COLUMN_BAD_FORMAT, _name, _style);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -101,22 +86,46 @@ void ColumnRecur::render (
   int width,
   Color& color)
 {
-  if (_style == "default" ||
-      _style == "duration")
+  if (task.has (_name))
   {
-    lines.push_back (
-      color.colorize (
-        rightJustify (
-          Duration (task.get ("recur")).formatCompact (),
-          width)));
-  }
-  else if (_style == "indicator")
-  {
-    if (task.has (_name))
-      lines.push_back (
-        color.colorize (
-          rightJustify (context.config.get ("recurrence.indicator"), width)));
+    if (_style == "default" ||
+        _style == "duration")
+      renderStringRight (lines, width, color, Duration (task.get (_name)).formatISO ());
+
+    else if (_style == "indicator")
+      renderStringRight (lines, width, color, Context::getContext ().config.get ("recurrence.indicator"));
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// The duration is stored in raw form, but it must still be valid,
+// and therefore is parsed first.
+void ColumnRecur::modify (Task& task, const std::string& value)
+{
+  // Try to evaluate 'value'.  It might work.
+  Variant evaluatedValue;
+  try
+  {
+    Eval e;
+    e.addSource (domSource);
+    contextTask = task;
+    e.evaluateInfixExpression (value, evaluatedValue);
+  }
+
+  catch (...)
+  {
+    evaluatedValue = Variant (value);
+  }
+
+  if (evaluatedValue.type () == Variant::type_duration)
+  {
+    // Store the raw value, for 'recur'.
+    std::string label = "  [1;37;43mMODIFICATION[0m ";
+    Context::getContext ().debug (label + _name + " <-- '" + value + '\'');
+    task.set (_name, value);
+  }
+  else
+    throw format ("The duration value '{1}' is not supported.", value);
+}
+
+////////////////////////////////////////////////////////////////////////////////
