@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,37 +20,35 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #include <cmake.h>
+#include <ColTags.h>
 #include <algorithm>
 #include <Context.h>
-#include <ColTags.h>
-#include <text.h>
-#include <i18n.h>
+#include <Eval.h>
+#include <Variant.h>
+#include <Filter.h>
+#include <shared.h>
+#include <format.h>
 #include <utf8.h>
+#include <main.h>
 
-extern Context context;
+extern Task& contextTask;
 
 ////////////////////////////////////////////////////////////////////////////////
 ColumnTags::ColumnTags ()
 {
   _name      = "tags";
-  _type      = "string";
   _style     = "list";
-  _label     = STRING_COLUMN_LABEL_TAGS;
+  _label     = "Tags";
   _styles    = {"list", "indicator", "count"};
-  _examples  = {STRING_COLUMN_EXAMPLES_TAGS,
-                context.config.get ("tag.indicator"),
+  _examples  = {"home @chore next",
+                Context::getContext ().config.get ("tag.indicator"),
                 "[2]"};
-  _hyphenate = context.config.getBoolean ("hyphenate");
-}
-
-////////////////////////////////////////////////////////////////////////////////
-ColumnTags::~ColumnTags ()
-{
+  _hyphenate = false;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -58,15 +56,15 @@ ColumnTags::~ColumnTags ()
 // Note that you can not determine which gets called first.
 void ColumnTags::setStyle (const std::string& value)
 {
-  _style = value;
+  Column::setStyle (value);
 
   if (_style == "indicator" &&
-      _label == STRING_COLUMN_LABEL_TAGS)
-    _label = _label.substr (0, context.config.get ("tag.indicator").length ());
+      _label == "Tags")
+    _label = _label.substr (0, Context::getContext ().config.get ("tag.indicator").length ());
 
   else if (_style == "count" &&
-            _label == STRING_COLUMN_LABEL_TAGS)
-    _label = STRING_COLUMN_LABEL_TAG;
+            _label == "Tags")
+    _label = "Tag";
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -74,15 +72,11 @@ void ColumnTags::setStyle (const std::string& value)
 void ColumnTags::measure (Task& task, unsigned int& minimum, unsigned int& maximum)
 {
   minimum = maximum = 0;
-
   if (task.has (_name))
   {
     if (_style == "indicator")
     {
-      if (task.has ("tags"))
-        minimum = maximum = utf8_width (context.config.get ("tag.indicator"));
-      else
-        minimum = maximum = 0;
+      minimum = maximum = utf8_width (Context::getContext ().config.get ("tag.indicator"));
     }
     else if (_style == "count")
     {
@@ -92,22 +86,25 @@ void ColumnTags::measure (Task& task, unsigned int& minimum, unsigned int& maxim
              _style == "list")
     {
       std::string tags = task.get (_name);
-      maximum = utf8_width (tags);
 
-      if (maximum)
+      // Find the widest tag.
+      if (tags.find (',') != std::string::npos)
       {
-        std::vector <std::string> all;
-        split (all, tags, ',');
-        for (auto& i : all)
+        auto all = split (tags, ',');
+        for (const auto& tag : all)
         {
-          unsigned int length = utf8_width (i);
+          auto length = utf8_width (tag);
           if (length > minimum)
             minimum = length;
         }
+
+        maximum = utf8_width (tags);
       }
+
+      // No need to split a single tag.
+      else
+        minimum = maximum = utf8_width (tags);
     }
-    else
-      throw format (STRING_COLUMN_BAD_FORMAT, _name, _style);
   }
 }
 
@@ -124,35 +121,67 @@ void ColumnTags::render (
     if (_style == "default" ||
         _style == "list")
     {
-      std::vector <std::string> allTags;
-      split (allTags, tags, ',');
-      if (allTags.size () > 1)
+      if (tags.find (',') != std::string::npos)
       {
-        std::sort (allTags.begin (), allTags.end ());
-        join (tags, " ", allTags);
+        auto all = split (tags, ',');
+        std::sort (all.begin (), all.end ());
+        tags = join (" ", all);
+
+        all.clear ();
+        wrapText (all, tags, width, _hyphenate);
+
+        for (const auto& i : all)
+          renderStringLeft (lines, width, color, i);
       }
-
-      std::vector <std::string> all;
-      wrapText (all, tags, width, _hyphenate);
-
-      for (auto& i : all)
-        lines.push_back (color.colorize (leftJustify (i, width)));
+      else
+        renderStringLeft (lines, width, color, tags);
     }
     else if (_style == "indicator")
     {
-      lines.push_back (
-        color.colorize (
-          rightJustify (context.config.get ("tag.indicator"), width)));
+      renderStringRight (lines, width, color, Context::getContext ().config.get ("tag.indicator"));
     }
     else if (_style == "count")
     {
-      std::vector <std::string> all;
-      split (all, tags, ',');
-      lines.push_back (
-        color.colorize (
-          rightJustify ("[" + format ((int)all.size ()) + "]", width)));
+      auto all = split (tags, ',');
+      renderStringRight (lines, width, color, '[' + format (static_cast <int> (all.size ())) + ']');
     }
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+void ColumnTags::modify (Task& task, const std::string& value)
+{
+  std::string label = "  [1;37;43mMODIFICATION[0m ";
+
+  // TW-1701
+  task.set (_name, "");
+
+  for (auto& tag : split (value, ','))
+  {
+    // If it's a DOM ref, eval it first.
+    Lexer lexer (tag);
+    std::string domRef;
+    Lexer::Type type;
+    if (lexer.token (domRef, type) &&
+        type == Lexer::Type::dom)
+    {
+      Eval e;
+      e.addSource (domSource);
+      contextTask = task;
+
+      Variant v;
+      e.evaluateInfixExpression (value, v);
+      task.addTag ((std::string) v);
+      Context::getContext ().debug (label + "tags <-- '" + (std::string) v + "' <-- '" + tag + '\'');
+    }
+    else
+    {
+      task.addTag (tag);
+      Context::getContext ().debug (label + "tags <-- '" + tag + '\'');
+    }
+
+    feedback_special_tags (task, tag);
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
