@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,27 +20,24 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #ifndef INCLUDED_COLUDA
 #define INCLUDED_COLUDA
-#define L10N                                           // Localization complete.
 
-#include <vector>
-#include <string>
-#include <Column.h>
-#include <Color.h>
-#include <Task.h>
+#include <ColTypeString.h>
+#include <ColTypeNumeric.h>
+#include <ColTypeDate.h>
+#include <ColTypeDuration.h>
 
-class ColumnUDA : public Column
+////////////////////////////////////////////////////////////////////////////////
+class ColumnUDAString : public ColumnTypeString
 {
 public:
-  ColumnUDA ();
-  ~ColumnUDA ();
-
-  bool validate (std::string&);
+  ColumnUDAString ();
+  bool validate (const std::string&) const;
   void measure (Task&, unsigned int&, unsigned int&);
   void render (std::vector <std::string>&, Task&, int, Color&);
 
@@ -52,5 +48,44 @@ private:
   bool _hyphenate;
 };
 
+////////////////////////////////////////////////////////////////////////////////
+class ColumnUDANumeric : public ColumnTypeNumeric
+{
+public:
+  ColumnUDANumeric ();
+  bool validate (const std::string&) const;
+  void measure (Task&, unsigned int&, unsigned int&);
+  void render (std::vector <std::string>&, Task&, int, Color&);
+
+public:
+  std::vector <std::string> _values;
+};
+
+////////////////////////////////////////////////////////////////////////////////
+class ColumnUDADate : public ColumnTypeDate
+{
+public:
+  ColumnUDADate ();
+  bool validate (const std::string&) const;
+  void measure (Task&, unsigned int&, unsigned int&);
+  void render (std::vector <std::string>&, Task&, int, Color&);
+
+public:
+  std::vector <std::string> _values;
+};
+
+////////////////////////////////////////////////////////////////////////////////
+class ColumnUDADuration : public ColumnTypeDuration
+{
+public:
+  ColumnUDADuration ();
+  bool validate (const std::string&) const;
+  void measure (Task&, unsigned int&, unsigned int&);
+  void render (std::vector <std::string>&, Task&, int, Color&);
+
+public:
+  std::vector <std::string> _values;
+};
+
 #endif
 ////////////////////////////////////////////////////////////////////////////////
