@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,55 +20,33 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
-#include <math.h>
-#include <Context.h>
+#include <cmake.h>
 #include <ColUUID.h>
-#include <text.h>
-#include <i18n.h>
-
-extern Context context;
+#include <format.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 ColumnUUID::ColumnUUID ()
 {
   _name       = "uuid";
-  _type       = "string";
   _style      = "long";
-  _label      = STRING_COLUMN_LABEL_UUID;
+  _label      = "UUID";
   _modifiable = false;
-
-  _styles.push_back ("long");
-  _styles.push_back ("short");
-
-  _examples.push_back ("f30cb9c3-3fc0-483f-bfb2-3bf134f00694");
-  _examples.push_back ("f30cb9c3");
-}
-
-////////////////////////////////////////////////////////////////////////////////
-ColumnUUID::~ColumnUUID ()
-{
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool ColumnUUID::validate (std::string& value)
-{
-  return true;
+  _styles     = {"long", "short"};
+  _examples   = {"f30cb9c3-3fc0-483f-bfb2-3bf134f00694", "f30cb9c3"};
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set the minimum and maximum widths for the value.
 void ColumnUUID::measure (Task&, unsigned int& minimum, unsigned int& maximum)
 {
+  // Mandatory attribute, no need to check the value.
+
        if (_style == "default" || _style == "long") minimum = maximum = 36;
   else if (_style == "short")                       minimum = maximum = 8;
-  else
-    throw format (STRING_COLUMN_BAD_FORMAT, _name, _style);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -79,14 +56,16 @@ void ColumnUUID::render (
   int width,
   Color& color)
 {
+  // No need to check the presence of UUID - all tasks have one.
+
   // f30cb9c3-3fc0-483f-bfb2-3bf134f00694  default
   // f30cb9c3                              short
   if (_style == "default" ||
       _style == "long")
-    lines.push_back (color.colorize (leftJustify (task.get (_name), width)));
+    renderStringLeft (lines, width, color, task.get (_name));
 
   else if (_style == "short")
-    lines.push_back (color.colorize (leftJustify (task.get (_name).substr (0, 8), width)));
+    renderStringLeft (lines, width, color, task.get (_name).substr (0, 8));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
