@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,55 +20,34 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
-#include <Context.h>
+#include <cmake.h>
 #include <ColUrgency.h>
-#include <text.h>
-#include <i18n.h>
-
-extern Context context;
+#include <format.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 ColumnUrgency::ColumnUrgency ()
 {
-  _name  = "urgency";
-  _type  = "number";
-  _style = "real";
-  _label = STRING_COLUMN_LABEL_URGENCY;
-
-  _styles.push_back ("real");
-  _styles.push_back ("integer");
-
-  _examples.push_back ("4.6");
-  _examples.push_back ("4");
-}
-
-////////////////////////////////////////////////////////////////////////////////
-ColumnUrgency::~ColumnUrgency ()
-{
+  _name       = "urgency";
+  _style      = "real";
+  _label      = "Urgency";
+  _modifiable = false;
+  _styles     = {"real", "integer"};
+  _examples   = {"4.6", "4"};
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set the minimum and maximum widths for the value.
 void ColumnUrgency::measure (Task& task, unsigned int& minimum, unsigned int& maximum)
 {
-  if (_style == "default" ||
-      _style == "real")
-  {
+  if (_style == "default" || _style == "real")
     minimum = maximum = format (task.urgency (), 4, 3).length ();
-  }
-  else if (_style == "integer")
-  {
-    minimum = maximum = format ((int)task.urgency ()).length ();
-  }
 
-  else
-    throw format (STRING_COLUMN_BAD_FORMAT, _name, _style);
+  else if (_style == "integer")
+    minimum = maximum = format ((int)task.urgency ()).length ();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -79,21 +57,11 @@ void ColumnUrgency::render (
   int width,
   Color& color)
 {
-  if (_style == "default" ||
-      _style == "real")
-  {
-    lines.push_back (
-      color.colorize (
-        rightJustify (
-          format (task.urgency (), 4, 3), width)));
-  }
+  if (_style == "default" || _style == "real")
+    renderDouble (lines, width, color, task.urgency ());
+
   else if (_style == "integer")
-  {
-    lines.push_back (
-      color.colorize (
-        rightJustify (
-          format ((int)task.urgency ()), width)));
-  }
+    renderInteger (lines, width, color, static_cast <int> (task.urgency ()));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
