@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,23 +20,19 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #ifndef INCLUDED_COLWAIT
 #define INCLUDED_COLWAIT
-#define L10N                                           // Localization complete.
 
-#include <ColDate.h>
+#include <ColTypeDate.h>
 
-class ColumnWait : public ColumnDate
+class ColumnWait : public ColumnTypeDate
 {
 public:
   ColumnWait ();
-  ~ColumnWait ();
-
-  bool validate (std::string&);
 };
 
 #endif
