@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,14 +20,15 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
-#include <Context.h>
+#include <cmake.h>
 #include <Column.h>
+#include <algorithm>
+#include <set>
+#include <Context.h>
 #include <ColDepends.h>
 #include <ColDescription.h>
 #include <ColDue.h>
@@ -39,23 +39,19 @@
 #include <ColMask.h>
 #include <ColModified.h>
 #include <ColParent.h>
-#include <ColPriority.h>
 #include <ColProject.h>
 #include <ColRecur.h>
 #include <ColScheduled.h>
 #include <ColStart.h>
 #include <ColStatus.h>
-#include <ColString.h>
 #include <ColTags.h>
 #include <ColUntil.h>
 #include <ColUrgency.h>
 #include <ColUUID.h>
 #include <ColUDA.h>
 #include <ColWait.h>
-#include <text.h>
-#include <i18n.h>
-
-extern Context context;
+#include <shared.h>
+#include <format.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 // Supports the complete column definition:
@@ -65,7 +61,7 @@ extern Context context;
 Column* Column::factory (const std::string& name, const std::string& report)
 {
   // Decompose name into type and style.
-  std::string::size_type dot = name.find ('.');
+  auto dot = name.find ('.');
   std::string column_name;
   std::string column_style;
   if (dot != std::string::npos)
@@ -90,7 +86,6 @@ Column* Column::factory (const std::string& name, const std::string& report)
   else if (column_name == "mask")        c = new ColumnMask ();
   else if (column_name == "modified")    c = new ColumnModified ();
   else if (column_name == "parent")      c = new ColumnParent ();
-  else if (column_name == "priority")    c = new ColumnPriority ();
   else if (column_name == "project")     c = new ColumnProject ();
   else if (column_name == "recur")       c = new ColumnRecur ();
   else if (column_name == "scheduled")   c = new ColumnScheduled ();
@@ -102,15 +97,12 @@ Column* Column::factory (const std::string& name, const std::string& report)
   else if (column_name == "uuid")        c = new ColumnUUID ();
   else if (column_name == "wait")        c = new ColumnWait ();
 
-  // Special non-task column.
-  else if (column_name == "string")      c = new ColumnString ();
-
   // UDA.
-  else if (context.config.get ("uda." + column_name + ".type") != "")
+  else if (Context::getContext ().config.has ("uda." + column_name + ".type"))
     c = Column::uda (column_name);
 
   else
-    throw format (STRING_COLUMN_BAD_NAME, column_name);
+    throw format ("Unrecognized column name '{1}'.", column_name);
 
   c->setReport (report);
   c->setStyle (column_style);
@@ -133,7 +125,6 @@ void Column::factory (std::map <std::string, Column*>& all)
   c = new ColumnMask ();           all[c->_name] = c;
   c = new ColumnModified ();       all[c->_name] = c;
   c = new ColumnParent ();         all[c->_name] = c;
-  c = new ColumnPriority ();       all[c->_name] = c;
   c = new ColumnProject ();        all[c->_name] = c;
   c = new ColumnRecur ();          all[c->_name] = c;
   c = new ColumnScheduled ();      all[c->_name] = c;
@@ -151,58 +142,78 @@ void Column::factory (std::map <std::string, Column*>& all)
 ////////////////////////////////////////////////////////////////////////////////
 void Column::uda (std::map <std::string, Column*>& all)
 {
-  // For each UDA, instantiate and initialize ColumnUDA().
-  std::map <std::string, int> udas;
-  std::vector <std::string> names;
-  context.config.all (names);
+  // For each UDA, instantiate and initialize ColumnUDA.
+  std::set <std::string> udas;
 
-  std::vector <std::string>::iterator i;
-  for (i = names.begin (); i != names.end (); ++i)
+  for (const auto& i : Context::getContext ().config)
   {
-    if (i->substr (0, 4) == "uda.")
+    if (i.first.substr (0, 4) == "uda.")
     {
-      std::string::size_type period = 4;
-      if ((period = i->find ('.', period)) != std::string::npos)
-        udas[i->substr (4, period - 4)] = 0;
+      std::string::size_type period = 4; // One byte after the first '.'.
+
+      if ((period = i.first.find ('.', period)) != std::string::npos)
+        udas.insert (i.first.substr (4, period - 4));
     }
   }
 
-  std::map <std::string, int>::iterator uda;
-  for (uda = udas.begin (); uda != udas.end (); ++uda)
+  for (const auto& uda : udas)
   {
-    if (all.find (uda->first) != all.end ())
-      throw format (STRING_UDA_COLLISION, uda->first);
+    if (all.find (uda) != all.end ())
+      throw format ("The UDA named '{1}' is the same as a core attribute, and is not permitted.", uda);
 
-    Column* c = Column::uda (uda->first);
-    all[c->_name] = c;
+    Column* c = Column::uda (uda);
+    if (c)
+      all[c->_name] = c;
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Column* Column::uda (const std::string& name)
 {
-  ColumnUDA* c = new ColumnUDA ();
-  c->_name = name;
+  auto type   = Context::getContext ().config.get ("uda." + name + ".type");
+  auto label  = Context::getContext ().config.get ("uda." + name + ".label");
+  auto values = Context::getContext ().config.get ("uda." + name + ".values");
 
-  std::string key = "uda." + name + ".type";
-  c->_type = context.config.get (key);
-  if (c->_type == "")
-    throw format (STRING_UDA_TYPE_MISSING, name);
-  if (c->_type != "string"   &&
-      c->_type != "date"     &&
-      c->_type != "duration" &&
-      c->_type != "numeric")
-    throw std::string (STRING_UDA_TYPE);
+  if (type == "string")
+  {
+    auto c = new ColumnUDAString ();
+    c->_name = name;
+    c->_label = label;
+    if (values != "")
+      c->_values = split (values, ',');
+    return c;
+  }
+  else if (type == "numeric")
+  {
+    auto c = new ColumnUDANumeric ();
+    c->_name = name;
+    c->_label = label;
+    if (values != "")
+      c->_values = split (values, ',');
+    return c;
+  }
+  else if (type == "date")
+  {
+    auto c = new ColumnUDADate ();
+    c->_name = name;
+    c->_label = label;
+    if (values != "")
+      c->_values = split (values, ',');
+    return c;
+  }
+  else if (type == "duration")
+  {
+    auto c = new ColumnUDADuration ();
+    c->_name = name;
+    c->_label = label;
+    if (values != "")
+      c->_values = split (values, ',');
+    return c;
+  }
+  else if (type != "")
+    throw std::string ("User defined attributes may only be of type 'string', 'date', 'duration' or 'numeric'.");
 
-  key = "uda." + name + ".label";
-  if (context.config.get (key) != "")
-    c->_label = context.config.get (key);
-
-  key = "uda." + name + ".values";
-  if (context.config.get (key) != "")
-    split (c->_values, context.config.get (key), ',');
-
-  return c;
+  return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -213,47 +224,11 @@ Column::Column ()
 , _label ("")
 , _report ("")
 , _modifiable (true)
+, _uda (false)
+, _fixed_width (false)
 {
 }
 
-////////////////////////////////////////////////////////////////////////////////
-Column::Column (const Column& other)
-{
-  _name       = other._name;
-  _type       = other._type;
-  _style      = other._style;
-  _label      = other._label;
-  _label      = other._report;
-  _modifiable = other._modifiable;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-Column& Column::operator= (const Column& other)
-{
-  if (this != &other)
-  {
-    _name        = other._name;
-    _type        = other._type;
-    _style       = other._style;
-    _label       = other._label;
-    _report      = other._report;
-    _modifiable  = other._modifiable;
-  }
-
-  return *this;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool Column::operator== (const Column& other) const
-{
-  return _name       == other._name   &&
-         _type       == other._type   &&
-         _style      == other._style  &&
-         _label      == other._label  &&
-         _report     == other._report &&
-         _modifiable == other._modifiable;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 Column::~Column ()
 {
@@ -265,8 +240,8 @@ void Column::renderHeader (
   int width,
   Color& color)
 {
-  if (context.verbose ("label") &&
-     nontrivial (_label))
+  if (Context::getContext ().verbose ("label") &&
+      _label != "")
   {
     // Create a basic label.
     std::string header;
@@ -277,8 +252,8 @@ void Column::renderHeader (
     Color c = color;
 
     // Now underline the header, or add a dashed line.
-    if (context.color () &&
-        context.config.getBoolean ("fontunderline"))
+    if (Context::getContext ().color () &&
+        Context::getContext ().config.getBoolean ("fontunderline"))
     {
       c.blend (Color (Color::nocolor, Color::nocolor, true, false, false));
       lines.push_back (c.colorize (leftJustify (header, width)));
@@ -292,37 +267,64 @@ void Column::renderHeader (
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Column::validate (std::string& input)
+void Column::setStyle (const std::string& style)
 {
-  return input.length () ? true : false;
+  if (style != "default" &&
+      std::find (_styles.begin (), _styles.end (), style) == _styles.end ())
+    throw format ("Unrecognized column format '{1}.{2}'", _name, style);
+
+  _style = style;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// No L10N.
-void Column::measure (const std::string&, unsigned int&, unsigned int&)
+// All integer values are right-justified.
+void Column::renderInteger (
+  std::vector <std::string>& lines,
+  int width,
+  Color& color,
+  int value)
 {
-  throw std::string ("Virtual method Column::measure not overridden.");
+  lines.push_back (
+    color.colorize (
+      rightJustify (value, width)));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// No L10N.
-void Column::measure (Task&, unsigned int&, unsigned int&)
+// All floating point values are right-justified.
+void Column::renderDouble (
+  std::vector <std::string>& lines,
+  int width,
+  Color& color,
+  double value)
 {
-  throw std::string ("Virtual method Column::measure not overridden.");
+  lines.push_back (
+    color.colorize (
+      rightJustify (
+        format (value, 4, 3), width)));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// No L10N.
-void Column::render (std::vector <std::string>&, const std::string&, int, Color&)
+void Column::renderStringLeft (
+  std::vector <std::string>& lines,
+  int width,
+  Color& color,
+  const std::string& value)
 {
-  throw std::string ("Virtual method Column::render not overridden.");
+  lines.push_back (
+    color.colorize (
+      leftJustify (value, width)));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// No L10N.
-void Column::render (std::vector <std::string>&, Task&, int, Color&)
+void Column::renderStringRight (
+  std::vector <std::string>& lines,
+  int width,
+  Color& color,
+  const std::string& value)
 {
-  throw std::string ("Virtual method Column::render not overridden.");
+  lines.push_back (
+    color.colorize (
+      rightJustify (value, width)));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
