@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,13 +20,12 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #ifndef INCLUDED_COLUMN
 #define INCLUDED_COLUMN
-#define L10N                                           // Localization complete.
 
 #include <vector>
 #include <string>
@@ -43,28 +41,35 @@ public:
   static Column* uda (const std::string&);
 
   Column ();
-  Column (const Column&);
-  Column& operator= (const Column&);
-  bool operator== (const Column&) const;     // TODO Is this necessary?
   virtual ~Column ();
 
-  std::string style () const                  { return _style;      }
-  std::string label () const                  { return _label;      }
-  std::string type () const                   { return _type;       }
-  bool modifiable () const                    { return _modifiable; }
-  std::vector <std::string> styles () const   { return _styles;     }
-  std::vector <std::string> examples () const { return _examples;   }
+  const std::string& name () const            { return _name;        }
+  const std::string& style () const           { return _style;       }
+  const std::string& label () const           { return _label;       }
+  const std::string& type () const            { return _type;        }
+  bool modifiable () const                    { return _modifiable;  }
+  bool is_uda () const                        { return _uda;         }
+  bool is_fixed_width () const                { return _fixed_width; }
+  std::vector <std::string> styles () const   { return _styles;      }
+  std::vector <std::string> examples () const { return _examples;    }
 
-  virtual void setStyle  (const std::string& value) { _style = value;  }
+  virtual void setStyle  (const std::string&);
   virtual void setLabel  (const std::string& value) { _label = value;  }
   virtual void setReport (const std::string& value) { _report = value; }
 
-  virtual bool validate (std::string&);
-  virtual void measure (const std::string&, unsigned int&, unsigned int&);
-  virtual void measure (Task&, unsigned int&, unsigned int&);
+  virtual void measure (const std::string&, unsigned int&, unsigned int&)           {};
+  virtual void measure (Task&, unsigned int&, unsigned int&)                        {};
   virtual void renderHeader (std::vector <std::string>&, int, Color&);
-  virtual void render (std::vector <std::string>&, const std::string&, int, Color&);
-  virtual void render (std::vector <std::string>&, Task&, int, Color&);
+  virtual void render (std::vector <std::string>&, const std::string&, int, Color&) {};
+  virtual void render (std::vector <std::string>&, Task&, int, Color&)              {};
+  virtual bool validate (const std::string&) const                                  {return false;};
+  virtual void modify (Task&, const std::string&)                                   {};
+
+protected:
+  void renderInteger     (std::vector <std::string>&, int, Color&, int);
+  void renderDouble      (std::vector <std::string>&, int, Color&, double);
+  void renderStringLeft  (std::vector <std::string>&, int, Color&, const std::string&);
+  void renderStringRight (std::vector <std::string>&, int, Color&, const std::string&);
 
 protected:
   std::string _name;
@@ -73,6 +78,8 @@ protected:
   std::string _label;
   std::string _report;
   bool _modifiable;
+  bool _uda;
+  bool _fixed_width;
   std::vector <std::string> _styles;
   std::vector <std::string> _examples;
 };
