@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -29,7 +29,7 @@
 
 #include <string>
 #include <vector>
-#include <Date.h>
+#include <Datetime.h>
 #include <Task.h>
 #include <Command.h>
 
@@ -40,7 +40,7 @@ public:
   int execute (std::string&);
 
 private:
-  std::string renderMonths (int, int, const Date&, std::vector <Task>&, int);
+  std::string renderMonths (int, int, const Datetime&, std::vector <Task>&, int);
 };
 
 #endif
