@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,31 +20,34 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
+#include <cmake.h>
+#include <CmdColor.h>
 #include <sstream>
-#include <ViewText.h>
+#include <Table.h>
 #include <Context.h>
 #include <main.h>
 #include <Color.h>
-#include <text.h>
-#include <i18n.h>
-#include <CmdColor.h>
-
-extern Context context;
+#include <format.h>
+#include <shared.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 CmdColor::CmdColor ()
 {
-  _keyword     = "colors";
-  _usage       = "task          colors [sample | legend]";
-  _description = STRING_CMD_COLOR_USAGE;
-  _read_only   = true;
-  _displays_id = false;
+  _keyword               = "colors";
+  _usage                 = "task          colors [sample | legend]";
+  _description           = "All colors, a sample, or a legend";
+  _read_only             = true;
+  _displays_id           = false;
+  _needs_gc              = false;
+  _uses_context          = false;
+  _accepts_filter        = false;
+  _accepts_modifications = false;
+  _accepts_miscellaneous = true;
+  _category              = Command::Category::misc;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -53,50 +55,46 @@ int CmdColor::execute (std::string& output)
 {
   int rc = 0;
 
-#ifdef FEATURE_COLOR
   // Get the non-attribute, non-fancy command line arguments.
-  bool legend = false;
-  std::vector <std::string> words = context.a3.extract_words ();
-  std::vector <std::string>::iterator word;
-  for (word = words.begin (); word != words.end (); ++word)
-    if (closeEnough ("legend", *word))
+  auto legend = false;
+  auto words = Context::getContext ().cli2.getWords ();
+  for (auto& word : words)
+    if (closeEnough ("legend", word))
       legend = true;
 
   std::stringstream out;
-  if (context.color ())
+  if (Context::getContext ().color ())
   {
     // If the description contains 'legend', show all the colors currently in
     // use.
     if (legend)
     {
-      out << "\n" << STRING_CMD_COLOR_HERE << "\n";
+      out << "\nHere are the colors currently in use:\n";
 
-      std::vector <std::string> all;
-      context.config.all (all);
+      Table view;
+      view.width (Context::getContext ().getWidth ());
+      if (Context::getContext ().config.getBoolean ("color"))
+        view.forceColor ();
+      view.add ("Color");
+      view.add ("Definition");
 
-      ViewText view;
-      view.width (context.getWidth ());
-      view.add (Column::factory ("string", STRING_CMD_COLOR_COLOR));
-      view.add (Column::factory ("string", STRING_CMD_COLOR_DEFINITION));
-
-      std::vector <std::string>::iterator item;
-      for (item = all.begin (); item != all.end (); ++item)
+      for (auto& item : Context::getContext ().config)
       {
         // Skip items with 'color' in their name, that are not referring to
         // actual colors.
-        if (*item != "_forcecolor" &&
-            *item != "color"       &&
-            item->find ("color") == 0)
+        if (item.first != "_forcecolor" &&
+            item.first != "color"       &&
+            item.first.find ("color") == 0)
         {
-          Color color (context.config.get (*item));
+          Color color (Context::getContext ().config.get (item.first));
           int row = view.addRow ();
-          view.set (row, 0, *item, color);
-          view.set (row, 1, context.config.get (*item), color);
+          view.set (row, 0, item.first, color);
+          view.set (row, 1, item.second, color);
         }
       }
 
       out << view.render ()
-          << "\n";
+          << '\n';
     }
 
     // If there is something in the description, then assume that is a color,
@@ -111,71 +109,69 @@ int CmdColor::execute (std::string& output)
       Color six    ("red on color173");
 
       std::string swatch;
-      for (word = words.begin (); word != words.end (); ++word)
+      for (auto word = words.begin (); word != words.end (); ++word)
       {
         if (word != words.begin ())
-          swatch += " ";
+          swatch += ' ';
 
         swatch += *word;
       }
 
       Color sample (swatch);
 
-      out << "\n"
-          << STRING_CMD_COLOR_EXPLANATION                                          << "\n"
+      out << '\n'
+          << "Use this command to see how colors are displayed by your terminal.\n"
           << "\n\n"
-          << STRING_CMD_COLOR_16                                                   << "\n"
-          << "  " << one.colorize ("task color black on bright yellow")            << "\n"
-          << "  " << two.colorize ("task color underline cyan on bright blue")     << "\n"
-          << "\n"
-          << STRING_CMD_COLOR_256                                                  << "\n"
-          << "  " << three.colorize ("task color color214 on color202")            << "\n"
-          << "  " << four.colorize ("task color rgb150 on rgb020")                 << "\n"
-          << "  " << five.colorize ("task color underline grey10 on grey3")        << "\n"
-          << "  " << six.colorize ("task color red on color173")                   << "\n"
-          << "\n"
-          << STRING_CMD_COLOR_YOURS                                                << "\n\n"
+          << "16-color usage (supports underline, bold text, bright background):\n"
+          << "  " << one.colorize ("task color black on bright yellow")            << '\n'
+          << "  " << two.colorize ("task color underline cyan on bright blue")     << '\n'
+          << '\n'
+          << "256-color usage (supports underline):\n"
+          << "  " << three.colorize ("task color color214 on color202")            << '\n'
+          << "  " << four.colorize ("task color rgb150 on rgb020")                 << '\n'
+          << "  " << five.colorize ("task color underline grey10 on grey3")        << '\n'
+          << "  " << six.colorize ("task color red on color173")                   << '\n'
+          << '\n'
+          << "Your sample:\n\n"
           << "  " << sample.colorize ("task color " + swatch)                      << "\n\n";
     }
 
     // Show all supported colors.  Possibly show some unsupported ones too.
     else
     {
-      out << "\n"
-          << STRING_CMD_COLOR_BASIC
-          << "\n"
-          << " " << Color::colorize (" black ",   "black")
-          << " " << Color::colorize (" red ",     "red")
-          << " " << Color::colorize (" blue ",    "blue")
-          << " " << Color::colorize (" green ",   "green")
-          << " " << Color::colorize (" magenta ", "magenta")
-          << " " << Color::colorize (" cyan ",    "cyan")
-          << " " << Color::colorize (" yellow ",  "yellow")
-          << " " << Color::colorize (" white ",   "white")
-          << "\n"
-          << " " << Color::colorize (" black ",   "white on black")
-          << " " << Color::colorize (" red ",     "white on red")
-          << " " << Color::colorize (" blue ",    "white on blue")
-          << " " << Color::colorize (" green ",   "black on green")
-          << " " << Color::colorize (" magenta ", "black on magenta")
-          << " " << Color::colorize (" cyan ",    "black on cyan")
-          << " " << Color::colorize (" yellow ",  "black on yellow")
-          << " " << Color::colorize (" white ",   "black on white")
+      out << '\n'
+          << "Basic colors\n"
+          << ' ' << Color::colorize (" black ",   "black")
+          << ' ' << Color::colorize (" red ",     "red")
+          << ' ' << Color::colorize (" blue ",    "blue")
+          << ' ' << Color::colorize (" green ",   "green")
+          << ' ' << Color::colorize (" magenta ", "magenta")
+          << ' ' << Color::colorize (" cyan ",    "cyan")
+          << ' ' << Color::colorize (" yellow ",  "yellow")
+          << ' ' << Color::colorize (" white ",   "white")
+          << '\n'
+          << ' ' << Color::colorize (" black ",   "white on black")
+          << ' ' << Color::colorize (" red ",     "white on red")
+          << ' ' << Color::colorize (" blue ",    "white on blue")
+          << ' ' << Color::colorize (" green ",   "black on green")
+          << ' ' << Color::colorize (" magenta ", "black on magenta")
+          << ' ' << Color::colorize (" cyan ",    "black on cyan")
+          << ' ' << Color::colorize (" yellow ",  "black on yellow")
+          << ' ' << Color::colorize (" white ",   "black on white")
           << "\n\n";
 
-      out << STRING_CMD_COLOR_EFFECTS
-          << "\n"
-          << " " << Color::colorize (" red ",               "red")
-          << " " << Color::colorize (" bold red ",          "bold red")
-          << " " << Color::colorize (" underline on blue ", "underline on blue")
-          << " " << Color::colorize (" on green ",          "black on green")
-          << " " << Color::colorize (" on bright green ",   "black on bright green")
-          << " " << Color::colorize (" inverse ",           "inverse")
+      out << "Effects\n"
+          << ' ' << Color::colorize (" red ",               "red")
+          << ' ' << Color::colorize (" bold red ",          "bold red")
+          << ' ' << Color::colorize (" underline on blue ", "underline on blue")
+          << ' ' << Color::colorize (" on green ",          "black on green")
+          << ' ' << Color::colorize (" on bright green ",   "black on bright green")
+          << ' ' << Color::colorize (" inverse ",           "inverse")
           << "\n\n";
 
       // 16 system colors.
       out << "color0 - color15"
-          << "\n"
+          << '\n'
           << "  0 1 2 . . .\n";
       for (int r = 0; r < 2; ++r)
       {
@@ -187,13 +183,13 @@ int CmdColor::execute (std::string& output)
           out << Color::colorize ("  ", s.str ());
         }
 
-        out << "\n";
+        out << '\n';
       }
 
       out << "          . . . 15\n\n";
 
       // Color cube.
-      out << STRING_CMD_COLOR_CUBE
+      out << "Color cube rgb"
           << Color::colorize ("0", "bold red")
           << Color::colorize ("0", "bold green")
           << Color::colorize ("0", "bold blue")
@@ -202,26 +198,26 @@ int CmdColor::execute (std::string& output)
           << Color::colorize ("5", "bold green")
           << Color::colorize ("5", "bold blue")
           << " (also color16 - color231)"
-          << "\n"
+          << '\n'
           << "  " << Color::colorize ("0            "
                                       "1            "
                                       "2            "
                                       "3            "
                                       "4            "
                                       "5", "bold red")
-          << "\n"
+          << '\n'
           << "  " << Color::colorize ("0 1 2 3 4 5  "
                                       "0 1 2 3 4 5  "
                                       "0 1 2 3 4 5  "
                                       "0 1 2 3 4 5  "
                                       "0 1 2 3 4 5  "
                                       "0 1 2 3 4 5", "bold blue")
-          << "\n";
+          << '\n';
 
       char label [12];
       for (int g = 0; g < 6; ++g)
       {
-        sprintf (label, " %d", g);
+        snprintf (label, 12, " %d", g);
         out << Color::colorize (label, "bold green");
         for (int r = 0; r < 6; ++r)
         {
@@ -232,17 +228,16 @@ int CmdColor::execute (std::string& output)
             out << Color::colorize ("  ", s.str ());
           }
 
-          out << " ";
+          out << ' ';
         }
 
-        out << "\n";
+        out << '\n';
       }
 
-      out << "\n";
+      out << '\n';
 
       // Grey ramp.
-      out << STRING_CMD_COLOR_RAMP
-          << " gray0 - gray23 (also color232 - color255)\n"
+      out << "Gray ramp gray0 - gray23 (also color232 - color255)\n"
           << "  0 1 2 . . .                             . . . 23\n"
           << "  ";
       for (int g = 0; g < 24; ++g)
@@ -253,21 +248,17 @@ int CmdColor::execute (std::string& output)
       }
 
       out << "\n\n"
-          << format (STRING_CMD_COLOR_TRY, "task color white on red")
-          << "\n\n";
+          << "Try running 'task color white on red'.\n"
+          << '\n';
     }
   }
   else
   {
-    out << STRING_CMD_COLOR_OFF << "\n";
+    out << "Color is currently turned off in your .taskrc file.  To enable color, remove the line 'color=off', or change the 'off' to 'on'.\n";
     rc = 1;
   }
 
   output = out.str ();
-#else
-  output = "Color not supported.\n";
-#endif
-
   return rc;
 }
 
