@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,58 +20,59 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
+#include <cmake.h>
+#include <CmdDuplicate.h>
 #include <iostream>
 #include <Context.h>
-#include <text.h>
+#include <Filter.h>
+#include <format.h>
 #include <util.h>
-#include <i18n.h>
 #include <main.h>
-#include <CmdDuplicate.h>
-
-extern Context context;
 
 ////////////////////////////////////////////////////////////////////////////////
 CmdDuplicate::CmdDuplicate ()
 {
-  _keyword     = "duplicate";
-  _usage       = "task <filter> duplicate <mods>";
-  _description = STRING_CMD_DUPLICATE_USAGE;
-  _read_only   = false;
-  _displays_id = false;
+  _keyword               = "duplicate";
+  _usage                 = "task <filter> duplicate <mods>";
+  _description           = "Duplicates the specified tasks";
+  _read_only             = false;
+  _displays_id           = false;
+  _needs_gc              = false;
+  _uses_context          = true;
+  _accepts_filter        = true;
+  _accepts_modifications = true;
+  _accepts_miscellaneous = false;
+  _category              = Command::Category::operation;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int CmdDuplicate::execute (std::string& output)
+int CmdDuplicate::execute (std::string&)
 {
-  int rc = 0;
-  int count = 0;
+  auto rc = 0;
+  auto count = 0;
 
   // Apply filter.
+  Filter filter;
   std::vector <Task> filtered;
-  filter (filtered);
+  filter.subset (filtered);
   if (filtered.size () == 0)
   {
-    context.footnote (STRING_FEEDBACK_NO_TASKS_SP);
+    Context::getContext ().footnote ("No tasks specified.");
     return 1;
   }
 
-  // Apply the command line modifications to the new task.
-  A3 modifications = context.a3.extract_modifications ();
-
   // Accumulated project change notifications.
   std::map <std::string, std::string> projectChanges;
 
-  std::vector <Task>::iterator task;
-  for (task = filtered.begin (); task != filtered.end (); ++task)
+  for (auto& task : filtered)
   {
     // Duplicate the specified task.
-    Task dup (*task);
+    Task dup (task);
+    dup.id = 0;                    // Reset, and TDB2::add will set.
     dup.set ("uuid", uuid ());     // Needs a new UUID.
     dup.remove ("start");          // Does not inherit start date.
     dup.remove ("end");            // Does not inherit end date.
@@ -85,44 +85,48 @@ int CmdDuplicate::execute (std::string& output)
       dup.remove ("recur");
       dup.remove ("until");
       dup.remove ("imask");
-
-      std::cout << format (STRING_CMD_DUPLICATE_NON_REC, task->id)
-          << "\n";
+      std::cout << format ("Note: task {1} was a recurring task.  The duplicated task is not.", task.identifier ())
+          << '\n';
     }
 
     // When duplicating a parent task, create a new parent task.
     else if (dup.getStatus () == Task::recurring)
     {
       dup.remove ("mask");
-
-      std::cout << format (STRING_CMD_DUPLICATE_REC, task->id)
-          << "\n";
+      std::cout << format ("Note: task {1} was a parent recurring task.  The duplicated task is too.", task.identifier ())
+          << '\n';
     }
 
     dup.setStatus (Task::pending); // Does not inherit status.
                                    // Must occur after Task::recurring check.
 
-    modify_task_annotate (dup, modifications);
+    dup.modify (Task::modAnnotate);
 
-    if (permission (dup,
-                    format (STRING_CMD_DUPLICATE_CONFIRM,
-                            task->id,
-                            task->get ("description")),
+    if (permission (format ("Duplicate task {1} '{2}'?",
+                            task.identifier (true),
+                            task.get ("description")),
                     filtered.size ()))
     {
-      context.tdb2.add (dup);
+      Context::getContext ().tdb2.add (dup);
       ++count;
-      feedback_affected (STRING_CMD_DUPLICATE_TASK, *task);
+      feedback_affected ("Duplicated task {1} '{2}'.", task);
 
-      if (context.verbose ("new-id"))
-        std::cout << format (STRING_CMD_ADD_FEEDBACK, context.tdb2.next_id ()) + "\n";
+      auto status = dup.getStatus ();
+      if (Context::getContext ().verbose ("new-id") &&
+          (status == Task::pending ||
+           status == Task::waiting))
+        std::cout << format ("Created task {1}.\n", dup.id);
 
-      if (context.verbose ("project"))
-        projectChanges[task->get ("project")] = onProjectChange (*task);
+      else if (Context::getContext ().verbose ("new-uuid") &&
+               status != Task::recurring)
+        std::cout << format ("Created task {1}.\n", dup.get ("uuid"));
+
+      if (Context::getContext ().verbose ("project"))
+        projectChanges[task.get ("project")] = onProjectChange (task);
     }
     else
     {
-      std::cout << STRING_CMD_DUPLICATE_NO << "\n";
+      std::cout << "Task not duplicated.\n";
       rc = 1;
       if (_permission_quit)
         break;
@@ -130,13 +134,12 @@ int CmdDuplicate::execute (std::string& output)
   }
 
   // Now list the project changes.
-  std::map <std::string, std::string>::iterator i;
-  for (i = projectChanges.begin (); i != projectChanges.end (); ++i)
-    if (i->first != "")
-      context.footnote (i->second);
+  for (const auto& change : projectChanges)
+    if (change.first != "")
+      Context::getContext ().footnote (change.second);
+
+  feedback_affected (count == 1 ? "Duplicated {1} task." : "Duplicated {1} tasks.", count);
 
-  context.tdb2.commit ();
-  feedback_affected (count == 1 ? STRING_CMD_DUPLICATE_1 : STRING_CMD_DUPLICATE_N, count);
   return rc;
 }
 
