@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,49 +20,36 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
+#include <cmake.h>
+#include <CmdExec.h>
 #include <stdlib.h>
 #include <Context.h>
-#include <i18n.h>
-#include <CmdExec.h>
-
-extern Context context;
+#include <shared.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 CmdExec::CmdExec ()
 {
-  _keyword     = "execute";
-  _usage       = "task          execute <external command>";
-  _description = STRING_CMD_EXEC_USAGE;
-  _read_only   = true;
-  _displays_id = false;
+  _keyword               = "execute";
+  _usage                 = "task          execute <external command>";
+  _description           = "Executes external commands and scripts";
+  _read_only             = true;
+  _displays_id           = false;
+  _needs_gc              = false;
+  _uses_context          = false;
+  _accepts_filter        = false;
+  _accepts_modifications = false;
+  _accepts_miscellaneous = true;
+  _category              = Command::Category::misc;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int CmdExec::execute (std::string& output)
+int CmdExec::execute (std::string&)
 {
-  std::string command_line;
-  std::vector <Arg>::iterator arg;
-  for (arg = context.a3.begin (); arg != context.a3.end (); ++arg)
-  {
-    if (arg != context.a3.begin () &&
-        arg->_raw != "execute" &&
-        arg->_category != Arg::cat_rc &&
-        arg->_category != Arg::cat_override)
-    {
-      if (command_line.length ())
-        command_line += " ";
-
-      command_line += "'" + arg->_raw + "'";
-    }
-  }
-
-  return system (command_line.c_str ());
+  return system (join (" ", Context::getContext ().cli2.getWords ()).c_str ());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
