@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,35 +20,32 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
+#include <cmake.h>
 #include <CmdLogo.h>
 #include <Context.h>
-#include <text.h>
-#include <i18n.h>
-
-extern Context context;
+#include <util.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 CmdLogo::CmdLogo ()
 {
-  _keyword     = "logo";
-  _usage       = "task          logo";
-  _description = STRING_CMD_LOGO_USAGE;
-  _read_only   = true;
-  _displays_id = false;
+  _keyword               = "logo";
+  _usage                 = "task          logo";
+  _description           = "Displays the Taskwarrior logo";
+  _read_only             = true;
+  _displays_id           = false;
+  _needs_gc              = false;
+  _uses_context          = false;
+  _accepts_filter        = false;
+  _accepts_modifications = false;
+  _accepts_miscellaneous = false;
+  _category              = Command::Category::misc;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Algorithm:
-//   Copy file rc.data.location/extensions
-//   Generate UUID
-//   Call the "install" function once, store results in rc:
-//     extension.<uuid>=<JSON>
 int CmdLogo::execute (std::string& output)
 {
   static const char* data[] =
@@ -85,10 +81,10 @@ int CmdLogo::execute (std::string& output)
     ""
   };
 
-  if (!context.color ())
-    throw std::string (STRING_CMD_LOGO_COLOR_REQ);
+  if (! Context::getContext ().color ())
+    throw std::string ("The logo command requires that color support is enabled.");
 
-  std::string indent (context.config.getInteger ("indent.report"), ' ');
+  std::string indent (Context::getContext ().config.getInteger ("indent.report"), ' ');
   output += optionalBlankLine ();
 
   for (int line = 0; data[line][0]; ++line)
@@ -104,7 +100,7 @@ int CmdLogo::execute (std::string& output)
       {
         value += 167;
         char block [24];
-        sprintf (block, "\033[48;5;%dm  \033[0m", value);
+        snprintf (block, 24, "\033[48;5;%dm  \033[0m", value);
         output += block;
       }
     }
@@ -118,12 +114,12 @@ int CmdLogo::execute (std::string& output)
       {
         value += 167;
         char block [24];
-        sprintf (block, "\033[48;5;%dm  \033[0m", value);
+        snprintf (block, 24, "\033[48;5;%dm  \033[0m", value);
         output += block;
       }
     }
 
-    output += "\n";
+    output += '\n';
   }
 
   output += optionalBlankLine ();
