@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,63 +20,58 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
+#include <cmake.h>
+#include <CmdUrgency.h>
 #include <sstream>
 #include <stdlib.h>
 #include <Context.h>
+#include <Filter.h>
+#include <Lexer.h>
 #include <main.h>
-#include <text.h>
-#include <i18n.h>
-#include <CmdUrgency.h>
-
-extern Context context;
+#include <format.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 CmdUrgency::CmdUrgency ()
 {
-  _keyword     = "_urgency";
-  _usage       = "task <filter> _urgency";
-  _description = STRING_CMD_URGENCY_USAGE;
-  _read_only   = true;
-  _displays_id = false;
+  _keyword               = "_urgency";
+  _usage                 = "task <filter> _urgency";
+  _description           = "Displays the urgency measure of a task";
+  _read_only             = true;
+  _displays_id           = false;
+  _needs_gc              = true;
+  _uses_context          = false;
+  _accepts_filter        = true;
+  _accepts_modifications = false;
+  _accepts_miscellaneous = false;
+  _category              = Command::Category::internal;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 int CmdUrgency::execute (std::string& output)
 {
   // Apply filter.
+  Filter filter;
   std::vector <Task> filtered;
-  filter (filtered);
+  filter.subset (filtered);
 
   if (filtered.size () == 0)
   {
-    context.footnote (STRING_FEEDBACK_NO_TASKS_SP);
+    Context::getContext ().footnote ("No tasks specified.");
     return 1;
   }
 
   // Display urgency for the selected tasks.
   std::stringstream out;
-  std::vector <Task>::iterator task;
-  for (task = filtered.begin (); task != filtered.end (); ++task)
+  for (auto& task : filtered)
   {
-    if (task->id)
-    {
-      out << format (STRING_CMD_URGENCY_RESULT,
-                     task->id, task->urgency ())
-          << "\n";
-    }
-    else
-    {
-      std::string uuid = task->get ("uuid");
-      out << format (STRING_CMD_URGENCY_RESULT,
-                     uuid, task->urgency ())
-          << "\n";
-    }
+    out << format ("task {1} urgency {2}",
+                   task.identifier (),
+                   Lexer::trim (format (task.urgency (), 6, 3)))
+        << '\n';
   }
 
   output = out.str ();
