@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -28,9 +28,9 @@
 #include <iostream>
 #include <vector>
 #include <stdlib.h>
-#include <text.h>
+#include <shared.h>
+#include <format.h>
 #include <util.h>
-#include <i18n.h>
 #include <Command.h>
 #include <main.h>
 
@@ -70,6 +70,7 @@
 #include <CmdModify.h>
 #include <CmdPrepend.h>
 #include <CmdProjects.h>
+#include <CmdPurge.h>
 #include <CmdReports.h>
 #include <CmdShow.h>
 #include <CmdStart.h>
@@ -81,6 +82,7 @@
 #include <CmdTimesheet.h>
 #include <CmdUDAs.h>
 #include <CmdUndo.h>
+#include <CmdUnique.h>
 #include <CmdUrgency.h>
 #include <CmdVersion.h>
 
@@ -88,8 +90,6 @@
 #include <ColProject.h>
 #include <ColDue.h>
 
-extern Context context;
-
 ////////////////////////////////////////////////////////////////////////////////
 void Command::factory (std::map <std::string, Command*>& all)
 {
@@ -105,6 +105,7 @@ void Command::factory (std::map <std::string, Command*>& all)
   c = new CmdCalendar ();           all[c->keyword ()] = c;
   c = new CmdColor ();              all[c->keyword ()] = c;
   c = new CmdColumns ();            all[c->keyword ()] = c;
+  c = new CmdCommands ();           all[c->keyword ()] = c;
   c = new CmdCompletionAliases ();  all[c->keyword ()] = c;
   c = new CmdCompletionColumns ();  all[c->keyword ()] = c;
   c = new CmdCompletionCommands (); all[c->keyword ()] = c;
@@ -130,9 +131,13 @@ void Command::factory (std::map <std::string, Command*>& all)
 #endif
   c = new CmdExport ();             all[c->keyword ()] = c;
   c = new CmdGet ();                all[c->keyword ()] = c;
+  c = new CmdGHistoryDaily ();      all[c->keyword ()] = c;
+  c = new CmdGHistoryWeekly ();     all[c->keyword ()] = c;
   c = new CmdGHistoryMonthly ();    all[c->keyword ()] = c;
   c = new CmdGHistoryAnnual ();     all[c->keyword ()] = c;
   c = new CmdHelp ();               all[c->keyword ()] = c;
+  c = new CmdHistoryDaily ();       all[c->keyword ()] = c;
+  c = new CmdHistoryWeekly ();      all[c->keyword ()] = c;
   c = new CmdHistoryMonthly ();     all[c->keyword ()] = c;
   c = new CmdHistoryAnnual ();      all[c->keyword ()] = c;
   c = new CmdIDs ();                all[c->keyword ()] = c;
@@ -143,6 +148,7 @@ void Command::factory (std::map <std::string, Command*>& all)
   c = new CmdModify ();             all[c->keyword ()] = c;
   c = new CmdPrepend ();            all[c->keyword ()] = c;
   c = new CmdProjects ();           all[c->keyword ()] = c;
+  c = new CmdPurge ();              all[c->keyword ()] = c;
   c = new CmdReports ();            all[c->keyword ()] = c;
   c = new CmdShow ();               all[c->keyword ()] = c;
   c = new CmdShowRaw ();            all[c->keyword ()] = c;
@@ -155,6 +161,7 @@ void Command::factory (std::map <std::string, Command*>& all)
   c = new CmdTimesheet ();          all[c->keyword ()] = c;
   c = new CmdUDAs ();               all[c->keyword ()] = c;
   c = new CmdUndo ();               all[c->keyword ()] = c;
+  c = new CmdUnique ();             all[c->keyword ()] = c;
   c = new CmdUrgency ();            all[c->keyword ()] = c;
   c = new CmdUUIDs ();              all[c->keyword ()] = c;
   c = new CmdVersion ();            all[c->keyword ()] = c;
@@ -165,12 +172,12 @@ void Command::factory (std::map <std::string, Command*>& all)
 
   // Instantiate a command object for each custom report.
   std::vector <std::string> reports;
-  for (auto &i : context.config)
+  for (auto &i : Context::getContext ().config)
   {
     if (i.first.substr (0, 7) == "report.")
     {
       std::string report = i.first.substr (7);
-      std::string::size_type columns = report.find (".columns");
+      auto columns = report.find (".columns");
       if (columns != std::string::npos)
         reports.push_back (report.substr (0, columns));
     }
@@ -180,71 +187,54 @@ void Command::factory (std::map <std::string, Command*>& all)
   {
     // Make sure a custom report does not clash with a built-in command.
     if (all.find (report) != all.end ())
-      throw format (STRING_CMD_CONFLICT, report);
+      throw format ("Custom report '{1}' conflicts with built-in task command.", report);
 
     c = new CmdCustom (
               report,
               "task <filter> " + report,
-              context.config.get ("report." + report + ".description"));
+              Context::getContext ().config.get ("report." + report + ".description"));
 
     all[c->keyword ()] = c;
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+const std::map <Command::Category, std::string> Command::categoryNames =
+{
+  // These strings are intentionally not l10n'd: they are used as identifiers.
+   {Command::Category::unassigned,   "unassigned"} // should never happen
+  ,{Command::Category::metadata,     "metadata"}
+  ,{Command::Category::report,       "report"}
+  ,{Command::Category::operation,    "operation"}
+  ,{Command::Category::context,      "context"}
+  ,{Command::Category::graphs,       "graphs"   }
+  ,{Command::Category::config,       "config"   }
+  ,{Command::Category::migration,    "migration"}
+  ,{Command::Category::misc,         "misc"     }
+  ,{Command::Category::internal,     "internal"}
+  ,{Command::Category::UNDOCUMENTED, "undocumented"}
+};
+
 ////////////////////////////////////////////////////////////////////////////////
 Command::Command ()
-: _usage ("")
+: _keyword ("")
+, _usage ("")
 , _description ("")
 , _read_only (true)
 , _displays_id (true)
 , _needs_confirm (false)
+, _needs_gc (true)
+, _uses_context (false)
+, _accepts_filter (false)
+, _accepts_modifications (false)
+, _accepts_miscellaneous (false)
+, _category(Category::unassigned)
 , _permission_quit (false)
 , _permission_all (false)
 , _first_iteration (true)
 {
 }
 
-////////////////////////////////////////////////////////////////////////////////
-Command::Command (const Command& other)
-{
-  _usage           = other._usage;
-  _description     = other._description;
-  _read_only       = other._read_only;
-  _displays_id     = other._displays_id;
-  _needs_confirm   = other._needs_confirm;
-  _permission_quit = other._permission_quit;
-  _permission_all  = other._permission_all;
-  _first_iteration = other._first_iteration;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-Command& Command::operator= (const Command& other)
-{
-  if (this != &other)
-  {
-    _usage           = other._usage;
-    _description     = other._description;
-    _read_only       = other._read_only;
-    _displays_id     = other._displays_id;
-    _needs_confirm   = other._needs_confirm;
-    _permission_quit = other._permission_quit;
-    _permission_all  = other._permission_all;
-    _first_iteration = other._first_iteration;
-  }
-
-  return *this;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool Command::operator== (const Command& other) const
-{
-  return _usage         == other._usage       &&
-         _description   == other._description &&
-         _read_only     == other._read_only   &&
-         _displays_id   == other._displays_id &&
-         _needs_confirm == other._needs_confirm;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 Command::~Command ()
 {
@@ -280,6 +270,42 @@ bool Command::displays_id () const
   return _displays_id;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+bool Command::needs_gc () const
+{
+  return _needs_gc;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+bool Command::uses_context () const
+{
+  return _uses_context;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+bool Command::accepts_filter () const
+{
+  return _accepts_filter;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+bool Command::accepts_modifications () const
+{
+  return _accepts_modifications;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+bool Command::accepts_miscellaneous () const
+{
+  return _accepts_miscellaneous;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+Command::Category Command::category () const
+{
+  return _category;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Returns true or false indicating whether to proceed with a write command, on
 // a per-task basis, after (potentially) asking for permission.
@@ -290,7 +316,6 @@ bool Command::displays_id () const
 //   rc.confirmation
 //   this->_read_only
 bool Command::permission (
-  const Task& task,
   const std::string& question,
   unsigned int quantity)
 {
@@ -307,8 +332,8 @@ bool Command::permission (
 
   // What remains are write commands that have not yet selected 'all' or 'quit'.
   // Describe the task.
-  bool         confirmation = context.config.getBoolean ("confirmation");
-  unsigned int bulk         = context.config.getInteger ("bulk");
+  bool         confirmation = Context::getContext ().config.getBoolean ("confirmation");
+  unsigned int bulk         = Context::getContext ().config.getInteger ("bulk");
 
   // Quantity 1 modifications have optional confirmation, and only (y/n).
   if (quantity == 1)
@@ -326,8 +351,8 @@ bool Command::permission (
   if ((bulk == 0 || quantity < bulk) && (!_needs_confirm || !confirmation))
     return true;
 
-  if (context.verbose ("blank") && !_first_iteration)
-    std::cout << "\n";
+  if (Context::getContext ().verbose ("blank") && !_first_iteration)
+    std::cout << '\n';
   int answer = confirm4 (question);
   _first_iteration = false;
   switch (answer)
