@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,46 +20,25 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
-#include <iostream>
-#include <cstring>
-#include <stdlib.h>
-
-#ifdef CYGWIN
-#include <time.h>
-#else
-#include <sys/time.h>
-#endif
-
-#include <i18n.h>
-#include <Context.h>
 #include <cmake.h>
-
-Context context;
-
-#ifdef HAVE_SRANDOM
-#define srand(x) srandom(x)
-#endif
+#include <iostream>
+#include <new>
+#include <cstring>
+#include <Context.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 int main (int argc, const char** argv)
 {
-  // Set up randomness.
-#ifdef CYGWIN
-  srand (time (NULL));
-#else
-  struct timeval tv;
-  gettimeofday (&tv, NULL);
-  srand (tv.tv_usec);
-#endif
+  int status {0};
 
-  int status = 0;
+  Context globalContext;
+  Context::setContext (&globalContext);
 
+  // Lightweight version checking that doesn't require initialization or any I/O.
   if (argc == 2 && !strcmp (argv[1], "--version"))
   {
     std::cout << VERSION << "\n";
@@ -69,9 +47,9 @@ int main (int argc, const char** argv)
   {
     try
     {
-      status = context.initialize (argc, argv);
+      status = Context::getContext ().initialize (argc, argv);
       if (status == 0)
-        status = context.run ();
+        status = Context::getContext ().run ();
     }
 
     catch (const std::string& error)
@@ -80,9 +58,15 @@ int main (int argc, const char** argv)
       status = -1;
     }
 
+    catch (std::bad_alloc& error)
+    {
+      std::cerr << "Error: Memory allocation failed: " << error.what () << "\n";
+      status = -3;
+    }
+
     catch (...)
     {
-      std::cerr << STRING_UNKNOWN_ERROR << "\n";
+      std::cerr << "Unknown error. Please report.\n";
       status = -2;
     }
   }
