@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,64 +20,33 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #ifndef INCLUDED_UTIL
 #define INCLUDED_UTIL
-#define L10N                                           // Localization complete.
 
+#include <cmake.h>
 #include <string>
 #include <vector>
 #include <map>
 #include <sys/types.h>
-#include <Task.h>
-#include <cmake.h>
-
-#ifdef HAVE_UUID
+#if defined(FREEBSD) || defined(OPENBSD)
+#include <uuid.h>
+#else
 #include <uuid/uuid.h>
 #endif
+#include <Table.h>
 
 // util.cpp
-bool confirm (const std::string&);
-int confirm3 (const std::string&);
 int confirm4 (const std::string&);
-void delay (float);
-std::string formatBytes (size_t);
-int autoComplete (const std::string&, const std::vector<std::string>&, std::vector<std::string>&, int minimum = 1);
 
-#if defined(HAVE_UUID) && !defined(HAVE_UUID_UNPARSE_LOWER)
+#ifndef HAVE_UUID_UNPARSE_LOWER
 void uuid_unparse_lower (uuid_t uu, char *out);
 #endif
 const std::string uuid ();
 
-int execute (const std::string&, std::vector<std::string>);
-
-#ifdef SOLARIS
-  #define LOCK_SH 1
-  #define LOCK_EX 2
-  #define LOCK_NB 4
-  #define LOCK_UN 8
-
-  int flock (int, int);
-#endif
-
-std::string compressIds (const std::vector <int>&);
-void combine (std::vector <int>&, const std::vector <int>&);
-
-unsigned burndown_size (unsigned ntasks);
-
-const std::string encode (const std::string&);
-const std::string decode (const std::string&);
-
-const std::string escape (const std::string&, char);
-
-const std::vector<std::string> indentTree (
-  const std::vector<std::string>&,
-  const std::string& whitespace = "  ",
-  char delimiter = '.');
-
 const std::string indentProject (
   const std::string&,
   const std::string& whitespace = "  ",
@@ -92,5 +60,9 @@ const std::vector <std::string> extractParents (
   time_t timegm (struct tm *tm);
 #endif
 
+bool nontrivial (const std::string&);
+const char* optionalBlankLine ();
+void setHeaderUnderline (Table&);
+
 #endif
 ////////////////////////////////////////////////////////////////////////////////
